#!/usr/bin/env perl

=head1 NAME

PrepareData -- a Perl program to prepare data for loading raw files

=head1 DESCRIPTION

Do something for you ...

=cut

BEGIN {
   use Env qw( APIIS_HOME );
   die "\n\tAPIIS_HOME is not set!\n\n" unless $APIIS_HOME;
   push @INC, "$APIIS_HOME/lib";
}

use strict;
use warnings;
use Apiis;
use Data::Dumper;
use Config::IniFiles;
use Spreadsheet::Read;
use Spreadsheet::WriteExcel;
use Archive::Tar;
use File::Find;

# command line parameters
use vars qw($opt_h $opt_d);
use Getopt::Std;
require apiis_alib;
Apiis->initialize;

use constant TAB 			=> "    ";
use constant OUTPUT_DIR 	=> "/tmp";

my $path;
my (%ms_markers, %my_dirs);

# script will start from here
run();

################################################################################

sub run{
	init();
	get_folders();
 	create_datafile();
}
# getting parameters and getting database connection
sub init{
	# read command line parameters
	getopts('hd:');
	usage() if $opt_h;		
	usage() if (!$opt_d);
	$path= $opt_d;
	print $path . "\n";
}

sub get_folders {
	my @folders;
	find sub {
		push @folders, "$File::Find::name" if (-d $File::Find::name);
	},"$path/raw_ms/raw_data/";
	foreach my $i (@folders){
# 		print "$i\n";
		$my_dirs{$i}++;
	}
}

sub create_datafile {
	opendir MYDIR, "$path/raw_ms/results";
	chdir "$path/raw_ms/results";
	my ($r,$c) = (1,1);
	# Create a new Excel workbook
	my $workbook = Spreadsheet::WriteExcel->new("$path/rawfile_ms.xls");
	my $format1 = $workbook->add_format();
    $format1->set_align('center');
	$format1->set_bold();
	my $format2 = $workbook->add_format();
    $format2->set_align('left');
	my $format3 = $workbook->add_format();
    $format3->set_align('left');
    $format3->set_color('red');

	my $worksheet = $workbook->add_worksheet();
	my $making_list = 1;
	my $row;
	while ($_ = readdir (MYDIR)){
		my $marker_name = $_;
		$marker_name =~ s/\..*//;
        $worksheet->write(0,$c,$marker_name,$format1);
		rename ($_,$marker_name.".csv");
		my $file_ref = ReadData("$path/raw_ms/results/$marker_name.csv",dtfmt =>"dd-mm-yyyy",sep =>"\t");
		if (($_ ne ".") and ($_ ne "..")){
			print  $marker_name.".csv\n";
			if ($making_list){
				$making_list = 0;
				$row = 2;
				$r = 1;
				while (1){
					my $dna = read_data($file_ref,$row,1,'yes');
					last if ($dna eq '');
					$worksheet->write($r,0,$dna,$format2);
					$row++;
					$r++;
				}
			}
			$ms_markers{$_} = $_;
			$row = 2;
			$r = 1;
			while (1){
				my $dna = read_data($file_ref,$row,1,'yes');
				last if ($dna eq '');
				my $dir = read_data($file_ref,$row,2,'yes');
				$dir =~ s/-/\//;
				my ($pos,$date);
				if ($dir ne ""){
					$pos = index($dir,"/");
					$dir = substr($dir,0,$pos+1) . substr($dir,$pos+1,4);
					$date = "";
					$date = substr($dir,$pos+1,2). "-".substr($dir,$pos+3,2);
				}
				my @years = ("01","02","03","04","05","06","07","08","09");
				my $found = 0;
				foreach (@years){
					$found = 0;
					if (dir_exist("$path/raw_ms/raw_data/$dir$_")){
						$worksheet->write($r,$c,"$dir$_",$format2);
						$date .= "-" . $_; 
						$found = 1;
						last;
					}
				}
				if (not $found){
					$worksheet->write($r,$c,$dir,$format3); # missing data
					$worksheet->write($r,$c+1,$date,$format3);
				} else {
					$worksheet->write($r,$c+1,$date,$format2);
				}
				$row++;
				$r++;
			}
			$c += 2;
		}
	}
}

sub dir_exist {
	my $dir = shift;
# 	print $dir . "\n";
	return 1 if defined $my_dirs{$dir};
	return 0;
}

sub trimstr{
	my $str = shift; $str =~ s/^\s+//; $str =~ s/\s+$//; return $str;
}


sub read_data {
	my ($datafile_ref,$row,$col,$trimming,$data_obj) = @_;
	my $datastr = $datafile_ref->[1]{cell}[$col][$row];
	if ($trimming eq 'yes'){
		$datastr = trimstr($datastr) if defined $datastr;	
	}	
	if (defined $datastr){
		return $datastr;
	} else {
		return "";
	}
}

=head1 AUTHOR

Truong Van Chi Cong <cong.chi@fli.bund.de>

=cut
