package Molab::Labels;

=head1 NAME

MolabIS::Labels - Translate all labels in MolabIS

=head1 DESCRIPTION

This module provides a translation for all web forms in MolabIS

=cut

##############################################################################
BEGIN {
    use Env qw( APIIS_HOME);
    die "\n\tAPIIS_HOME is not set!\n\n" unless $APIIS_HOME;
    push @INC, "$APIIS_HOME/lib";
}

use Apiis::Init;
use strict;
use warnings;
use Config::IniFiles;
##############################################################################

sub get_labels {
	my $section_name = shift;
    my %forms_desc;
    my $debug_path  = "/tmp/molab_debug_file";     # debug file

	# start debugging
	my $debug = 0;                        # 1: debug, 0: none
	if ($debug) { open DBG_FILE, ">>", $debug_path; }
	use POSIX qw(strftime);
	if ($debug) {
	    print DBG_FILE "\n---------------- start of request -----------------";
	    print DBG_FILE "\n" . strftime "%a %b %e %H:%M:%S %Y", localtime;
	}
	print DBG_FILE "\n" . "Generated from Labels.pm ...."  if $debug;
	# which language is used to translate labels
	my $cfg = new Config::IniFiles( -file => "../../etc/apiisrc" );
	if ($debug){
		print DBG_FILE "Language: " . $cfg->val( 'I18N', 'language' )
        	if $cfg->val( 'I18N', 'language' );
	}
	# get all values for a given section
	my %ini;
	my $infile;
	if ($cfg->val( 'I18N', 'language' )) {
	    $infile = "../../etc/language_".$cfg->val( 'I18N', 'language' ).".conf";
	} else {
		$infile = "../../etc/language_en.conf";
	}
	 
	tie %ini, 'Config::IniFiles', ( -file => $infile );
	my %my_section = %{$ini{$section_name}};
	for my $key ( keys %my_section ) {
		$forms_desc{$key}= $my_section{$key};	
		print DBG_FILE "\n" .$key . " = " . $forms_desc{$key}
		if ($debug);
	}
	
	# finish debugging 
	if ($debug) {
		print DBG_FILE "\n" . strftime "%a %b %e %H:%M:%S %Y", localtime;
	    print DBG_FILE "\n--------------- end of request -------------------\n";
	}
    return \%forms_desc;
}


=head1 AUTHORS

Truong Van Chi Cong <cong.chi@fli.bund.de>

=cut

1;
