package Molab::Params;

=head1 NAME

Molab::Params - Set parameters for web forms 

=head1 DESCRIPTION

This module allows the developer to set up all parameters for web forms.
These parameters are mainly the delaration of SQL strings used to extract 
data from MolabIS database. 

=cut

################################################################################
BEGIN {
    use Env qw( APIIS_HOME);
    die "\n\tAPIIS_HOME is not set!\n\n" unless $APIIS_HOME;
    push @INC, "$APIIS_HOME/lib";
}
use Apiis::Init;
use Data::Dumper;

use constant NO 	=> 0;
use constant YES	=> 1;
use constant FALSE 	=> 0;
use constant TRUE	=> 1;

################################################################################

=head2 get_filter_sql - Get a SQL string for the filter

This subroutine get a SQL string for a specified filter on a given form.
Basically, the SQL string is created without any parameters.

=over 4

=item B<input>:

    1. A string of form name
    2. A string of filter name

=item B<output>:

    1. A SQL string which is used to extract data for a filter

=back

=cut

sub get_filter_sql{
    my ($form_name,$filter_name) = @_;
    $apiis->log('info','Params::get_filter_sql('. $form_name. ',' .$filter_name. ')');
    my $sql = "";
    if ($form_name eq "codes"){
      if ($filter_name eq "filter_1_loop"){
        $sql = "SELECT DISTINCT class, codes.class 
                FROM codes
                WHERE class != 'ENTRY_ACTION' and
                      class != 'SQLACTION' and
                      class != 'EVENT' and
                      class != 'EXIT_ACTION' and
                      class != 'ENTRY_ACTION' and
                      class != 'ID_SET' and
                      class != 'DONOR_TYPE' and
                      class != 'HYBRID'
                ";
      }
    }
    elsif ($form_name eq "contacts"){
      if ($filter_name eq "filter_3_loop"){
      	$sql = "SELECT ext_code,short_name,db_code
                FROM codes
                WHERE class='COUNTRY'
                ORDER BY ext_code";
	  }
    }
    elsif ($form_name eq "protocols"){
      if ($filter_name eq "filter_1_loop"){
        $sql = "SELECT DISTINCT codes.ext_code, ext_code
                FROM codes 
                WHERE class='PROTOCOL_TYPE'";
      }
    }
    elsif ($form_name eq "markers"){
      if ($filter_name eq "filter_1_loop"){
        $sql = "SELECT DISTINCT ext_code, long_name 
                FROM codes WHERE class='MARKER_TYPE'";
      }
    }
    elsif ($form_name eq "projects"){
      if ($filter_name eq "filter_1_loop"){
        $sql = "SELECT DISTINCT project_id, project_name 
                FROM projects ";
      }
    }
    elsif ($form_name eq "organisms"){
      if ($filter_name eq "filter_2_loop"){
        $sql = "SELECT ext_code, codes.short_name
                FROM codes 
                WHERE class='SPECIES'";
      }
      if ($filter_name eq "filter_3_loop"){
        $sql = "SELECT ext_code, codes.short_name
                FROM codes 
                WHERE class='BREED'";
      }
    }
    elsif ($form_name eq "reuse_samples"){
        $sql = "SELECT ext_code, codes.short_name
                FROM codes 
                WHERE class='BREED'";
    }
    # Samples
    elsif ($form_name eq "samples"){
      if ($filter_name eq "filter_1_loop"){
        $sql = "SELECT DISTINCT project_id, projects.project_name
                FROM projects";
      }
	  elsif ($filter_name eq "filter_2_loop"){
        $sql = "SELECT DISTINCT ext_code, codes.short_name 
                FROM codes 
                WHERE class='MATERIAL_TYPE'
                AND ext_code != 'DNA'";
      }
    }
    # DNA
    elsif ($form_name eq "dna"){
      if ($filter_name eq "filter_1_loop"){
        $sql = "SELECT DISTINCT project_id, projects.project_name
                FROM projects";
      }
    }
    elsif ($form_name eq "electrophoresis"){
      if ($filter_name eq "filter_1_loop"){
        $sql = "SELECT project_id, project_name
                FROM projects";
      }
      if ($filter_name eq "filter_2_loop"){
        $sql = "SELECT electrophoresis_id, ext_electrophoresis
                FROM electrophoresises";
      }
      if ($filter_name eq "filter_3_loop"){
        $sql = "SELECT marker_id, marker_name
                FROM markers";
      }
    }
    elsif ($form_name eq "pcr"){
      if ($filter_name eq "filter_1_loop"){
        $sql = "SELECT project_id, project_name
                FROM projects";
      }
      if ($filter_name eq "filter_2_loop"){
        $sql = "SELECT amplification_id, ext_amplification
                FROM pcr_amplification";
      }
      if ($filter_name eq "filter_3_loop"){
        $sql = "SELECT marker_id, marker_name
                FROM markers";
      }
    }
    elsif ($form_name eq "microsatellites"){
      if ($filter_name eq "filter_3_loop"){
        $sql = "SELECT project_id, project_name
                FROM projects";
      }
      if ($filter_name eq "filter_4_loop"){
        $sql = "SELECT marker_id, marker_name
                FROM markers mk, codes cd
                WHERE mk.db_marker_type = cd.db_code
                AND cd.ext_code = 'Microsatellite'";
      }
    }
    elsif ($form_name eq "sequences"){
      if ($filter_name eq "filter_3_loop"){
        $sql = "SELECT project_id, project_name
                FROM projects";
      }
      if ($filter_name eq "filter_4_loop"){
        $sql = "SELECT marker_id, marker_name
                FROM markers mk, codes cd
                WHERE mk.db_marker_type = cd.db_code
                AND cd.ext_code = 'Sequencing'";
      }
    }
    # reporting
    elsif ($form_name eq "samples_by_project"){
      if ($filter_name eq "filter_1_loop"){
        $sql = "SELECT project_id, project_name 
                FROM projects";
      }
    }
    elsif ($form_name eq "dna_by_project"){
      if ($filter_name eq "filter_1_loop"){
        $sql = "SELECT project_id, project_name 
                FROM projects";
      }
    }
    elsif ($form_name eq "microsatellites_by_project"){
      if ($filter_name eq "filter_1_loop"){
        $sql = "SELECT project_id, project_name 
                FROM projects";
      }
    }
    elsif ($form_name eq "analysis_by_organism"){
      if ($filter_name eq "filter_1_loop"){
        $sql = "SELECT project_id, project_name 
                FROM projects";
      }
    }  
    elsif ($form_name eq "sequences_per_marker"){
      if ($filter_name eq "filter_1_loop"){
        $sql = "SELECT project_id, project_name 
                FROM projects";
      }
    }
    elsif ($form_name eq "microsatellites_per_marker"){
      if ($filter_name eq "filter_1_loop"){
        $sql = "SELECT project_id, project_name 
                FROM projects";
      }
    }
    elsif ($form_name eq "breeds_project"){
      if ($filter_name eq "filter_1_loop"){
        $sql = "SELECT project_id, project_name 
                FROM projects";
      }
    }
    return $sql;
}

sub make_filter {
    my ($form_name,$filter_name,$current_opt,$has_all) = @_;
    my $sql = Molab::Params::get_filter_sql($form_name,$filter_name);
    $apiis->log('debug', "Params::make_filter: " . $sql);
    my $sql_ref = $apiis->DataBase->user_sql($sql);
    $apiis->check_status;
    my @col_values = ( );
    # set the first option with value "All" to some specified forms
    if ($has_all eq 'yes'){
        my %first_row;
        $first_row{option_val} = "%";
        $first_row{option}     = "All";
        push(@col_values, \%first_row);
    }
    # loop for existing values
    while ( my @data_array = $sql_ref->handle->fetchrow_array ) {
        my %row_data;
        $row_data{option_val} = Encode::encode_utf8($data_array[0]);
        $row_data{option}     = $data_array[1];
        if (defined $current_opt and ($row_data{option} eq $current_opt)) {
            $row_data{selected} = ' selected="selected"';
        }
        push(@col_values, \%row_data);
    }
    return \@col_values;
}

=head2 get_list_sql - Return a SQL string for a given drop-down list 

This subroutine will return a SQL string used to extract the data for a given
drop-down list. 
For example, you can use the following syntax to get a SQL string for a 
drop-down list named db_country_loop on the form.
	
	$sql = Molab::Params::get_list_sql('db_country_loop');

=over 4

=item B<input>:

    String: the string of the drop-down list

=item B<output>:

    String: SQL string which is used to extract data for the drop-down list

=back

=cut

sub get_list_sql {
    my ($layout,$field_name) =@_;
    $apiis->log('info',"Params::get_list_sql(".$layout.",".$field_name.")");
    my $sql;
    
    # Contacts
    if ($layout eq 'contacts'){
	  if ($field_name eq "db_country_loop"){
      	$sql = "SELECT ext_code,short_name,db_code
                FROM codes
                WHERE class='COUNTRY'
                ORDER BY ext_code";
      }
      elsif($field_name eq "db_language_loop"){
        $sql = "SELECT ext_code,short_name,db_code
                FROM codes
                WHERE class='LANGUAGE'
                ORDER BY ext_code";
      }
    }
    # Markers
    elsif ($layout eq 'markers'){
      if($field_name eq "db_primer1_loop"){
        $sql = "SELECT ext_code,short_name,db_code
                FROM codes
                WHERE class='PRIMER'";
      }
      elsif($field_name eq "db_primer2_loop"){
        $sql = "SELECT ext_code,short_name,db_code
                FROM codes
                WHERE class='PRIMER'";
      }
    }
    # Organisms or Insert Organisms
    elsif (($layout eq 'organisms') or ($layout eq 'insert_organisms')) {
      if($field_name eq "db_species_loop"){
        $sql = "SELECT ext_code,short_name,db_code,long_name
                FROM codes
                WHERE class='SPECIES'
                ORDER BY short_name";
      }
      elsif($field_name eq "db_breed_loop"){
        $sql = "SELECT ext_code,short_name,db_code
                FROM codes
                WHERE class='BREED'
                ORDER BY short_name";
      }
      elsif($field_name eq "db_donor_type_loop"){
        $sql = "SELECT ext_code,short_name,db_code
                FROM codes
                WHERE class='DONOR_TYPE'
                ORDER BY short_name";
      }
      elsif($field_name eq "db_org_loop"){
        $sql = "SELECT ext_id,ext_id,db_unit
                FROM unit
                WHERE ext_unit='ORGANIZATION'";
      }
      elsif($field_name eq "db_sex_loop"){
        $sql = "SELECT ext_code,long_name,db_code
                FROM codes
                WHERE class='SEX'";
      }
      elsif($field_name eq "db_sire_loop"){
        $sql = "SELECT t.ext_organism,t.ext_organism,t.db_organism
                FROM transfer t
                WHERE t.db_organism=1
                UNION
                SELECT t.ext_organism,t.ext_organism,t.db_organism
                FROM transfer t
                  LEFT JOIN organisms a USING (db_organism)
                  LEFT JOIN codes c on a.db_sex=c.db_code
                WHERE c.ext_code='m'";
      }
      elsif($field_name eq "db_dam_loop"){
        $sql = "SELECT t.ext_organism,t.ext_organism,t.db_organism
                FROM transfer t
                WHERE t.db_organism=2
                UNION
                SELECT t.ext_organism,t.ext_organism,t.db_organism
                FROM transfer t
                  LEFT JOIN organisms a USING (db_organism)
                  LEFT JOIN codes c on a.db_sex=c.db_code
                WHERE c.ext_code='f'";
      }
    }
    # Samples or Collect Samples
    elsif (($layout eq 'samples') or ($layout eq 'collect_samples1') 
      or ($layout eq 'collect_samples2') or ($layout eq 'collect_samples3')) {
      if($field_name eq "project_name_loop"){
        $sql = "SELECT project_id, project_name, project_id
                FROM projects
                ORDER BY project_id DESC";
      }
      elsif($field_name eq "ext_organism_loop"){
        $sql = "SELECT ext_organism,ext_organism,db_organism
                FROM transfer
                WHERE ext_organism != 'unknown_sire'
                  AND ext_organism != 'unknown_dam'
                ORDER BY ext_organism ASC";
      }
      elsif($field_name eq "protocol_name_loop"){
        $sql = "SELECT protocol_id,protocol_name,protocol_id
                FROM protocols pt
                  LEFT JOIN codes cd on cd.db_code=pt.db_protocol_type
                WHERE cd.ext_code='Sampling'";
      }
      elsif($field_name eq "db_donor_type_loop"){
        $sql = "SELECT ext_code,short_name,db_code
                FROM codes
                WHERE class='DONOR_TYPE'";
      }
      elsif($field_name eq "db_material_type_loop"){
        $sql = "SELECT ext_code,short_name,db_code
                FROM codes
                WHERE class='MATERIAL_TYPE' 
                  AND ext_code != 'DNA'";
      }
      elsif($field_name eq "db_user_loop"){
        $sql = "SELECT c.db_contact,first_name||' '||second_name,c.db_contact
                FROM unit u, contacts c
                WHERE ext_unit='USERS' and 
                u.db_contact = c.db_contact";
      }
      elsif($field_name eq "sp1_0_loop"){
        $sql = "SELECT s.storage_id,s.storage_name,s.storage_id
                FROM storage s
                WHERE parent_id=0";
      } 
      elsif($field_name eq "dna_checking_protocol_loop"){
        $sql = "SELECT protocol_id,protocol_name,protocol_id
                FROM protocols";
      }
      elsif($field_name eq "db_vessel_type_loop"){
        $sql = "SELECT ext_code,short_name,db_code
                FROM codes
                WHERE class='VESSEL_TYPE'";
      }
    }
    # DNA and Extract DNA
    elsif (($layout eq 'dna') or ($layout eq 'extract_dna1') 
      or ($layout eq 'extract_dna2')) {
      if($field_name eq "project_name_loop"){
        $sql = "SELECT project_id, project_name, project_id
                FROM projects
                ORDER BY project_id DESC";
      }
      elsif($field_name eq "ext_organism_loop"){
        $sql = "SELECT ext_organism,ext_organism,db_organism
                FROM transfer
                WHERE ext_organism != 'unknown_sire'
                  AND ext_organism != 'unknown_dam'
                ORDER BY ext_organism ASC";
      }
      elsif($field_name eq "protocol_name_loop"){
        $sql = "SELECT protocol_id,protocol_name,protocol_id
                FROM protocols pt
                  LEFT JOIN codes cd on cd.db_code=pt.db_protocol_type
                WHERE cd.ext_code='DNA extraction'";
      }
      elsif($field_name eq "db_donor_type_loop"){
        $sql = "SELECT ext_code,short_name,db_code
                FROM codes
                WHERE class='DONOR_TYPE'";
      }
      elsif($field_name eq "db_material_type_loop"){
        $sql = "SELECT ext_code,short_name,db_code
                FROM codes
                WHERE class='MATERIAL_TYPE' 
                  AND ext_code != 'DNA'";
      }
      elsif($field_name eq "db_user_loop"){
        $sql = "SELECT c.db_contact,first_name||' '||second_name,c.db_contact
                FROM unit u, contacts c
                WHERE ext_unit='USERS' and 
                u.db_contact = c.db_contact";
      }
      elsif($field_name eq "sp1_0_loop"){
        $sql = "SELECT s.storage_id,s.storage_name,s.storage_id
                FROM storage s
                WHERE parent_id=0";
      } 
      elsif($field_name eq "dna_checking_protocol_loop"){
       $sql = "SELECT protocol_id,protocol_name,protocol_id
                FROM protocols pt
                  LEFT JOIN codes cd on cd.db_code=pt.db_protocol_type
                WHERE cd.ext_code='DNA validation'";
      }
      elsif($field_name eq "db_vessel_type_loop"){
        $sql = "SELECT ext_code,short_name,db_code
                FROM codes
                WHERE class='VESSEL_TYPE'";
      }
    }
    # Reuse Samples
    elsif ($layout eq 'reuse_samples') {
      if($field_name eq "destination_project_loop"){
        $sql = "SELECT project_id, project_name, project_id
                FROM projects
                ORDER BY project_id DESC";
      }
      elsif($field_name eq "target_project_loop"){
        $sql = "SELECT project_id, project_name, project_id
                FROM projects
                ORDER BY project_id DESC";
      }
      elsif($field_name eq "db_species_loop"){
        $sql = "SELECT ext_code,short_name,db_code,long_name
                FROM codes
                WHERE class='SPECIES'
                ORDER BY short_name";
      }
      elsif($field_name eq "db_breed_loop"){
        $sql = "SELECT ext_code,short_name,db_code
                FROM codes
                WHERE class='BREED'
                ORDER BY short_name";
      }
      elsif($field_name eq "db_material_type_loop"){
        $sql = "SELECT ext_code,short_name,db_code
                FROM codes
                WHERE class='MATERIAL_TYPE' 
                  AND ext_code != 'DNA'";
      }
    }
    # Prepare Samples
    elsif ($layout eq 'prepare_samples1') {
      if($field_name eq "project_name_loop"){
        $sql = "SELECT project_id, project_name, project_id
                FROM projects
                ORDER BY project_id DESC";
      }
      elsif($field_name eq "db_material_type_loop"){
        $sql = "SELECT ext_code,short_name,db_code
                FROM codes
                WHERE class='MATERIAL_TYPE' 
                  AND ext_code != 'DNA'";
      }
    }
    # Prepare Working Solutions
    elsif ($layout eq 'prepare_working_sol1') {
      if($field_name eq "project_name_loop"){
        $sql = "SELECT project_id, project_name, project_id
                FROM projects
                ORDER BY project_id DESC";
      }
    }
    # Amplify PCR
    elsif ($layout eq 'amplify_pcr1') {
      if($field_name eq "project_name_loop"){
        $sql = "SELECT project_id, project_name, project_id
                FROM projects
                ORDER BY project_id DESC";
      }
      elsif($field_name eq "protocol_name_loop"){
		$sql = "SELECT protocol_id,protocol_name,protocol_id
                FROM protocols pt
                  LEFT JOIN codes cd on cd.db_code=pt.db_protocol_type
                WHERE cd.ext_code='PCR amplification'";
      }
      elsif($field_name eq "markers_loop"){
        $sql = "SELECT marker_id,marker_name,marker_id
                FROM markers
				ORDER BY marker_name ";
      }
    }
    # Check PCR Products
    elsif ($layout eq 'check_pcr_products1') {
      if($field_name eq "project_name_loop"){
        $sql = "SELECT project_id, project_name, project_id
                FROM projects
                ORDER BY project_id DESC";
      }
      elsif($field_name eq "markers_loop"){
        $sql = "SELECT marker_id,marker_name,marker_id
                FROM markers";
      }
      elsif($field_name eq "pcr_amplification_loop"){
        $sql = "SELECT amplification_id,ext_amplification,amplification_id
                FROM pcr_amplification";
      }
    }
    # PCR
    elsif ($layout eq 'pcr') {
      if($field_name eq "project_name_loop"){
        $sql = "SELECT project_id, project_name, project_id
                FROM projects
                ORDER BY project_id DESC";
      }
      elsif($field_name eq "pcr_amplification_protocol_loop"){
        $sql = "SELECT protocol_id,protocol_name,protocol_id
                FROM protocols pt
                  LEFT JOIN codes cd on cd.db_code=pt.db_protocol_type
                WHERE cd.ext_code='PCR amplification'";
      }
      elsif($field_name eq "pcr_validation_protocol_loop"){
        $sql = "SELECT protocol_id,protocol_name,protocol_id
                FROM protocols pt
                  LEFT JOIN codes cd on cd.db_code=pt.db_protocol_type
                WHERE cd.ext_code='PCR validation'";
      }
      elsif($field_name eq "marker_loop"){
        $sql = "SELECT marker_id,marker_name,marker_id
                FROM markers";
      }
    }
    # Electrophoresis
    elsif (($layout eq 'electrophoresis') or ($layout eq 'run_electrophoresis1')) {
      if($field_name eq "project_name_loop"){
        $sql = "SELECT project_id, project_name, project_id
                FROM projects
                ORDER BY project_id DESC";
      }
      elsif($field_name eq "protocol_name_loop"){
        $sql = "SELECT protocol_id,protocol_name,protocol_id
                FROM protocols pt
                    LEFT JOIN codes cd on cd.db_code=pt.db_protocol_type
                WHERE cd.ext_code='Electrophoresis'";
      }
      elsif($field_name eq "pcr_name_loop"){
        $sql = "SELECT amplification_id,ext_amplification,amplification_id
                FROM pcr_amplification";
      }
      elsif($field_name eq "marker_loop"){
        $sql = "SELECT marker_id, marker_name, marker_id FROM markers mk";
      }
      elsif($field_name eq "protocol_loop"){
        $sql = "SELECT protocol_id,protocol_name,protocol_id
                FROM protocols pt
                    LEFT JOIN codes cd on cd.db_code=pt.db_protocol_type
                WHERE cd.ext_code='Electrophoresis'";
      }
      elsif($field_name eq "electrophoresis_purpose_loop"){
        $sql = "SELECT ext_code,short_name,db_code
                FROM codes
                WHERE class='PURPOSE'";
      }
      elsif($field_name eq "electrophoresis_method_loop"){
        $sql = "SELECT ext_code,short_name,db_code
                FROM codes
                WHERE class='METHOD'";
      }
      elsif($field_name eq "software_loop"){
        $sql = "SELECT ext_code,short_name,db_code
                FROM codes
                WHERE class='SOFTWARE'";
      }
      elsif($field_name eq "user_loop"){
        $sql = "SELECT c.db_contact,first_name,c.db_contact
                FROM unit u, contacts c
                WHERE ext_unit='USERS' and 
                u.db_contact = c.db_contact";
      }
    }
    # Load Electrophoresis products
    elsif (($layout eq 'load_electrophoresis1') or ($layout eq 'load_electrophoresis2')) {
      if($field_name eq "project_name_loop"){
        $sql = "SELECT project_id, project_name, project_id
                FROM projects
                ORDER BY project_id DESC";
      }
      elsif($field_name eq "ext_electrophoresis_loop"){
        $sql = "SELECT electrophoresis_id,ext_electrophoresis,electrophoresis_id
                FROM electrophoresises";
      }
      elsif($field_name eq "user_loop"){
        $sql = "SELECT c.db_contact,first_name,c.db_contact
                FROM unit u, contacts c
                WHERE ext_unit='USERS' and 
                u.db_contact = c.db_contact";
      }
      elsif($field_name eq "software_loop"){
        $sql = "SELECT ext_code,short_name,db_code
                FROM codes
                WHERE class='SOFTWARE'";
      }
    }
    # Analyze Sequencing
    elsif (($layout eq 'analyze_sequencing1') or ($layout eq 'analyze_sequencing2')) {
      if($field_name eq "project_name_loop"){
        $sql = "SELECT project_id, project_name, project_id
                FROM projects
                ORDER BY project_id DESC";
      }
      elsif($field_name eq "markers_loop"){
		$sql = "SELECT mk.marker_id,mk.marker_name,mk.marker_id
                FROM markers mk, codes cd 
                WHERE db_marker_type = db_code 
                  AND ext_code = 'Sequencing'";
      }
    }
    # Analyze Genotyping
    elsif (($layout eq 'analyze_genotyping1') or ($layout eq 'analyze_genotyping2')) {
      if($field_name eq "project_name_loop"){
        $sql = "SELECT project_id, project_name, project_id
                FROM projects
                ORDER BY project_id DESC";
      }
      elsif($field_name eq "markers_loop"){
		$sql = "SELECT mk.marker_id,mk.marker_name,mk.marker_id
                FROM markers mk, codes cd 
                WHERE db_marker_type = db_code 
                  AND ext_code = 'Microsatellite'";
      }
    }
    # Unaligned sequences formats
    elsif ($layout eq 'sequence_formats') {
      if($field_name eq "project_name_loop"){
        $sql = "SELECT project_id, project_name, project_id
                FROM projects
                ORDER BY project_id DESC";
      }
      elsif($field_name eq "marker_loop"){
        $sql = "SELECT marker_id,marker_name,marker_id
                FROM markers";
      }
    }
    # Aligned sequences formats
    elsif ($layout eq 'aligned_sequences') {
    }
	# Microsatellite ToolKit
    elsif ($layout eq 'ms_toolkit') {
      if($field_name eq "project_name_loop"){
        $sql = "SELECT project_id, project_name, project_id
                FROM projects
                ORDER BY project_id DESC";
      }
      elsif($field_name eq "marker_loop"){
        $sql = "SELECT mk.marker_id,mk.marker_name,mk.marker_id
                FROM markers mk, codes cd 
                WHERE db_marker_type = db_code 
                  AND ext_code = 'Microsatellite'";
      }
    }
    
    return $sql;
}

=head2 make_list - Make a drop-down list

This subroutine will extract data from the Codes table to prepare data for
creating a drop-down on the form. Normally, the result is used for filling
into a <TMPL_LOOP> element in a HTML template. 

=over 4

=item B<input>:

    1. String: a valid SQL string 
         (e.g. SELECT ext_code,short_name,db_code FROM codes WHERE class='COUNTRY')
    2. String: current value in the drop-down list
     

=item B<output>:

    1. Array Ref: point to the array of values

=back

=cut

sub make_list {
    my ($sql, $current_opt, $i, $flag, $all_option) = @_;
    $apiis->log('info', "Params::make_list:(".$sql.",".$current_opt.",".$i.")");
    my $sql_ref = $apiis->DataBase->user_sql($sql);
    $apiis->check_status;
    my @col_values = ( );
    # add a default option at the top of the option list
    if ($all_option eq 'no'){
	    my %default_row;
	    $default_row{option_val} = "-1";
	    $default_row{option}     = "select one";
	    push(@col_values,\%default_row);
    }
    if ($all_option eq 'yes'){
    	my %selectall;
    	$selectall{option_val} = "%";
    	$selectall{option}     = "All";
    	push(@col_values,\%selectall);	
    }
    # push all other options to the option list    
    while ( my @data_array = $sql_ref->handle->fetchrow_array ) {
        my %row_data;
# 		$row_data{option_val} = Encode::encode_utf8($data_array[0]);
	    #$row_data{option_val} = Encode::decode_utf8($data_array[0]);
		$row_data{option_val} = $data_array[0];
        $row_data{option}     = $data_array[1];
        if ($flag eq 1){
        	$row_data{option} .= ': ' .  $data_array[3];
        }
        #$row_data{option}     = Encode::decode_utf8( $data_array[1]);
        if ($data_array[$i] eq $current_opt) {
            $row_data{selected} = 'selected="selected"';
        }
        push(@col_values, \%row_data);
    }
    return \@col_values;
}

sub make_multilist {
    my ($sql, $current_opt, $i, $flag, $control) = @_;
    $apiis->log('info', "Params::make_multilist:(".$sql.",".$current_opt.",".$i.")");
    my $sql_ref = $apiis->DataBase->user_sql($sql);
    $apiis->check_status;
    my @col_values = ( );
#    # add a default option at the top of the option list
#    my %default_row;
#    $default_row{option_val} = "-1";
#    $default_row{option}     = "select one";
#    push(@col_values,\%default_row);
    # push all other options to the option list  
    if ($control eq "yes"){
    	my %select_more;
    	$select_more{option_val} = "%";
    	$select_more{option}     = "select one or more ...";
    	push(@col_values,\%select_more);	
    }  
    while ( my @data_array = $sql_ref->handle->fetchrow_array ) {
        my %row_data;
        $row_data{option_val} = $data_array[0];
	    #$row_data{option_val} = Encode::decode_utf8($data_array[0]);
        $row_data{option}     = $data_array[1];
        if ($flag eq 1){
        	$row_data{option} .= ': ' .  $data_array[3];
        }
        #$row_data{option}     = Encode::decode_utf8( $data_array[1]);
        if ($data_array[$i] eq $current_opt) {
            $row_data{selected} = 'selected="selected"';
        }
        push(@col_values, \%row_data);
    }
    return \@col_values;
}

=head2 get_search_sql - Get a SQL string for searching

This subroutine get a SQL string for searching a set of records. The SQL string
is formed from a specified action of a given layout. The form inputs of filters
are used to fill in the WHERE of SQL query.

=over 4

=item B<input>:

    1. A string of action name
    2. A string of layout name
    3. A reference to a hash of form inputs
    4. An identification of session

=item B<output>:

    1. A SQL string which is used to extract data for searching

=back

=cut

sub get_search_sql {
  my ($action, $layout, $form_input, $sid) = @_;
  my $sql = "";
  if ($layout eq "codes"){
  	if ($action eq "ajax_fill"){
      $sql  = "SELECT ";
      $sql .= "  guid        as guid, ";
      $sql .= "  class       as classname, ";
      $sql .= "  ext_code    as ext_code, ";
      $sql .= "  short_name  as short_name, ";
      $sql .= "  long_name   as long_name, ";
      $sql .= "  opening_dt  as opening_dt, ";
      $sql .= "  closing_dt  as closing_dt, ";
      $sql .= "  description as comment "; 
      $sql .= "FROM codes ";
      $sql .= sprintf "WHERE codes.guid::text = %s ",
              $apiis->DataBase->dbh->quote($form_input->{ID});;
    }
    elsif ($action eq "ajax_search"){
      $sql  = "SELECT ";
      $sql .= "  class       as class, ";
      $sql .= "	 short_name  as short_name, ";
      $sql .= "  long_name   as long_name, ";
      $sql .= "  guid        as guid ";
      $sql .= "FROM codes ";
      $sql .= sprintf "WHERE class= %s ",
              $apiis->DataBase->dbh->quote($form_input->{filter_1_loop});;
      $sql .= " ORDER BY $form_input->{'order_by'} $form_input->{'asc_desc'}";
    }
  }
  elsif ($layout eq "protocols"){
    if ($action eq "ajax_fill") {
      $sql  = "SELECT ";
      $sql .= "  p.guid          as guid, ";
      $sql .= "  p.protocol_id   as protocol_id, ";
      $sql .= "  c.ext_code      as db_protocol_type, ";
      $sql .= "  p.protocol_name as protocol_name, ";
      $sql .= "  p.comment       as comment "; 
      $sql .= "FROM protocols p, codes c ";
      $sql .= "WHERE c.db_code=p.db_protocol_type ";
      $sql .= "AND p.guid::text ='$form_input->{ID}'";
    }
    if ($action eq "ajax_search") {
      $sql  = "SELECT ";
      $sql .= "  p.guid          as guid, ";
      $sql .= "  protocol_name   as protocol_name, ";
      $sql .= "  ext_code        as db_protocol_type, ";
      $sql .= "  comment         as comment ";
      $sql .= "FROM  protocols p, codes c ";
      $sql .= sprintf "WHERE p.db_protocol_type=c.db_code AND c.ext_code= %s ",
              $apiis->DataBase->dbh->quote($form_input->{filter_1_loop});;
      $sql .= "ORDER BY $form_input->{'order_by'} $form_input->{'asc_desc'}";
    }
  }
  elsif ($layout eq "contacts"){
    if ($action eq "ajax_fill") {
      $sql  = "SELECT ";
      $sql .= "  u.guid        as unit__guid, ";
      $sql .= "  c.guid        as contacts__guid, ";
      $sql .= "  u.ext_unit    as ext_unit, ";
      $sql .= "  u.ext_id      as ext_id, ";
      $sql .= "  c.phone       as phone, ";
      $sql .= "  c.phone2      as phone2, ";
      $sql .= "  c.fax         as fax, ";
      $sql .= "  c.email       as email, ";
      $sql .= "  c.account     as account, ";
      $sql .= "  c.bank_name   as bank_name, ";
      $sql .= "  c.bank_zip    as bank_zip, ";
      $sql .= "  c.title       as title, ";
      $sql .= "  c.salutation  as salutation, ";
      $sql .= "  c.birth_dt    as birth_dt, ";
      $sql .= "  c.first_name  as first_name, ";
      $sql .= "  c.second_name as second_name, ";
      $sql .= "  c.third_name  as third_name, ";
      $sql .= "  c.db_language as db_language_loop_div, ";
      $sql .= "  c.street      as street, ";
      $sql .= "  c.town        as town, ";
      $sql .= "  c.zip         as zip, ";
      $sql .= "  c.db_country  as db_country_loop_div, ";
      $sql .= "  c.label       as label, ";
      $sql .= "  c.comment     as comment "; 
      $sql .= "FROM unit u LEFT JOIN contacts c  ON u.db_contact=c.db_contact";
      $sql .= sprintf "  WHERE u.guid::text = %s ",
              $apiis->DataBase->dbh->quote($form_input->{ID});
    }
    if ($action eq "ajax_search"){
      $sql  = "SELECT ";
      $sql .= "  un.ext_id      as ext_id, ";
      $sql .= "  ct.first_name  as first_name, ";
      $sql .= "  ct.second_name as second_name, ";
      $sql .= "  cd.long_name   as country, ";
	  $sql .= "  ct.town        as town, ";
      $sql .= "  ct.email       as email, ";
      $sql .= "  un.guid        as guid "; 
      $sql .= "FROM unit un LEFT JOIN contacts ct ON un.db_contact=ct.db_contact ";
	  $sql .= "  LEFT JOIN codes cd on ct.db_country=cd.db_code ";
	  $sql .= "WHERE un.ext_unit = 'USERS' ";
      $sql .= sprintf "  AND un.ext_id like %s ",
              $apiis->DataBase->dbh->quote($form_input->{filter_1}.'%')
			  if ($form_input->{filter_1} ne "");
	  $sql .= sprintf "  AND ct.first_name like %s ",
              $apiis->DataBase->dbh->quote($form_input->{filter_2}.'%')
			  if ($form_input->{filter_2} ne "");
      $sql .= sprintf "  AND ct.second_name like %s ",
              $apiis->DataBase->dbh->quote($form_input->{filter_3}.'%')
			  if ($form_input->{filter_3} ne "");
      $sql .= sprintf "  AND cd.short_name like %s ",
              $apiis->DataBase->dbh->quote($form_input->{filter_4_loop}.'%')
			  if (($form_input->{filter_4_loop} ne "") and
				  ($form_input->{filter_4_loop} ne "%"));
      $sql .= sprintf "  AND ct.town like %s ",
              $apiis->DataBase->dbh->quote($form_input->{filter_5}.'%')
			  if ($form_input->{filter_5} ne "");
      $sql .= "ORDER BY $form_input->{'order_by'} ";
      $sql .= " $form_input->{'asc_desc'}";
    }
  }
  if ($layout eq "markers"){
  	if ($action eq "ajax_fill"){
      $sql  = "SELECT ";
      $sql .= "  mk.guid        as guid, ";
      $sql .= "  c.ext_code     as marker_type, ";
      $sql .= "  mk.marker_name as marker_name, ";
      $sql .= "  mk.db_primer1  as db_primer1_loop_div, ";
      $sql .= "  mk.db_primer2  as db_primer2_loop_div, ";  
      $sql .= "  mk.comment     as comment "; 
      $sql .= "FROM markers mk ";
      $sql .= "  LEFT JOIN codes c ON mk.db_marker_type=c.db_code ";
      $sql .= sprintf "WHERE mk.guid::text = %s ",
              $apiis->DataBase->dbh->quote($form_input->{ID});;
    }
    elsif ($action eq "ajax_search"){
      $sql  = "SELECT ";
      $sql .= "	 c.short_name   as marker_type, ";
      $sql .= "  mk.marker_name as marker_name, ";
      $sql .= "  pm1.short_name as primer1, ";
      $sql .= "  pm2.short_name as primer2, ";
      $sql .= "  mk.guid        as guid ";
      $sql .= "FROM markers mk ";
      $sql .= "  LEFT JOIN codes c ON mk.db_marker_type=c.db_code ";
      $sql .= "  LEFT JOIN codes pm1 ON mk.db_primer1=pm1.db_code ";
      $sql .= "  LEFT JOIN codes pm2 ON mk.db_primer2=pm2.db_code ";
      $sql .= sprintf "WHERE c.ext_code= %s ",
              $apiis->DataBase->dbh->quote($form_input->{filter_1_loop});;
      $sql .= " ORDER BY $form_input->{'order_by'} $form_input->{'asc_desc'}";
    }
  }
  if ($layout eq "projects"){
    if ($action eq "ajax_fill"){
      $sql  = "SELECT ";
      $sql .= "  guid          as guid, ";
      $sql .= "  project_name  as project_name, ";
      $sql .= "  keyword       as keyword, ";
      $sql .= "  begin_dt      as begin_dt, ";
      $sql .= "	 finish_dt     as finish_dt, ";
      $sql .= "	 description   as description, ";
      $sql .= "	 comment       as comment ";
      $sql .= "FROM projects ";
      $sql .= sprintf "WHERE projects.guid::text = %s ",
              $apiis->DataBase->dbh->quote($form_input->{ID});;
    }
    if ($action eq "ajax_search"){
      $sql  = "SELECT ";
      $sql .= "  guid          as guid, ";
      $sql .= "  project_name  as project_name, ";
      $sql .= "  keyword       as keyword, ";
      $sql .= "  description   as description ";
      $sql .= "FROM projects ";
      $sql .= sprintf "WHERE project_id::text like %s ",
              $apiis->DataBase->dbh->quote($form_input->{filter_1_loop});;
      $sql .= "ORDER BY $form_input->{'order_by'} $form_input->{'asc_desc'}";
    }
  }
  elsif ($layout eq "organisms"){
    if ($action eq "ajax_fill") {
      $sql  = "SELECT ";
      $sql .= "  a.guid          as my_guid, ";
      $sql .= "  t.guid          as transfer__guid, ";
      $sql .= "  t.ext_organism  as ext_organism, ";
      $sql .= "  a.db_species    as db_species_loop_div, ";
      $sql .= "  a.db_breed      as db_breed_loop_div, ";
      $sql .= "  a.latitude      as latitude, ";
      $sql .= "  a.longitude     as longitude, ";
      $sql .= "  a.db_donor_type as db_donor_type_loop_div, ";
      $sql .= "  a.db_sire       as db_sire_loop_div, ";
      $sql .= "  a.db_dam        as db_dam_loop_div, ";
      $sql .= "  a.db_sex        as db_sex_loop_div, ";
      $sql .= "  a.birth_dt      as birth_dt, ";
      $sql .= "  a.birth_year    as birth_year, ";
      $sql .= "  a.comment       as comment, ";
      $sql .= "  a.db_photo      as image_id, ";
      $sql .= "  '<img width=\"370\" height=\"278\" ";
      $sql .= "  src=\"/cgi-bin/export.cgi?sid=".$sid.",'||a.db_photo||'\"/\>'  as organisms_image_div ";
      $sql .= "FROM organisms a left join codes c1 on ";
      $sql .= " (a.db_species=c1.db_code) left join codes c2 on ";
      $sql .= " (a.db_breed=c2.db_code), transfer t ";
      $sql .= "WHERE a.db_organism=t.db_organism ";
      $sql .= sprintf" AND a.guid::text = %s",
              $apiis->DataBase->dbh->quote($form_input->{ID});
    }
    if ($action eq "ajax_search") {
      $sql  = "SELECT ";
      $sql .= "  t.ext_organism as ext_organism, ";
      $sql .= "  c1.short_name as db_species, ";
      $sql .= "  c2.short_name as db_breed, ";
      $sql .= "  c3.long_name as db_sex, a.guid as guid ";
      $sql .= "FROM organisms a ";
      $sql .= "     left join codes c1 on (a.db_species=c1.db_code) ";
      $sql .= "     left join codes c2 on (a.db_breed=c2.db_code) ";
      $sql .= "     left join codes c3 on (a.db_sex=c3.db_code), transfer t ";
      $sql .= "WHERE a.db_organism=t.db_organism ";
      $sql .= sprintf "  AND t.ext_organism like %s ",
              $apiis->DataBase->dbh->quote($form_input->{filter_1}.'%');
      $sql .= sprintf "  AND c1.ext_code like %s ",
              $apiis->DataBase->dbh->quote($form_input->{filter_2_loop}.'%');
      $sql .= sprintf "  AND c2.ext_code like %s ",
              $apiis->DataBase->dbh->quote($form_input->{filter_3_loop}.'%');
      $sql .= "ORDER BY $form_input->{'order_by'} $form_input->{'asc_desc'}";
    }
  }
  elsif ($layout eq "samples"){
    if ($action eq "ajax_fill"){
      $sql  = "SELECT distinct ";
      $sql .= "  s.guid             as sample_guid, ";
      $sql .= "  p.project_id       as project_name_loop_div, ";
      $sql .= "  s.ext_sample       as ext_sample, ";
      $sql .= "  t.db_organism      as ext_organism_loop_div, ";
      $sql .= "  s.db_material_type as db_material_type_loop_div, ";
      $sql .= "  s.production_dt    as production_dt, ";
      $sql .= "  s.protocol_id      as protocol_name_loop_div, ";
      $sql .= "  s.storage_dt       as storage_dt, ";
      $sql .= "  s.db_vessel_type   as db_vessel_type_loop_div, ";
      $sql .= "  ss.no_units        as volume, ";
      $sql .= "  s.db_user          as db_user_loop_div, ";
      $sql .= "  s.comment          as comment, ";
      $sql .= "  s1.storage_id      as sp1_0_loop_div, ";
      $sql .= "  s2.storage_id      as sp1_1_loop_div, ";
      $sql .= "  s3.storage_id      as sp1_2_loop_div, ";
      $sql .= "  s4.storage_id      as sp1_3_loop_div, ";
      $sql .= "  s5.storage_name    as sp1_4 ";
      $sql .= "FROM samples s, projects p, transfer t, ";
      $sql .= "     sample_collection sc, storage_samples ss, storage s5 ";
      $sql .= "  INNER JOIN storage as s4 ON (s5.parent_id = s4.storage_id) ";
      $sql .= "  INNER JOIN storage as s3 ON (s4.parent_id = s3.storage_id) ";
      $sql .= "  INNER JOIN storage as s2 ON (s3.parent_id = s2.storage_id) ";
      $sql .= "  INNER JOIN storage as s1 ON (s2.parent_id = s1.storage_id) ";
      $sql .= "WHERE p.project_id  = sc.project_id ";
      $sql .= "  AND s.db_organism = t.db_organism  ";
      $sql .= "  AND sc.sample_id  = s.sample_id ";
      $sql .= "  AND s.sample_id   = ss.sample_id ";
      $sql .= "  AND ss.storage_id = s5.storage_id ";
      $sql .= sprintf "  AND s.guid::text = %s",
              $apiis->DataBase->dbh->quote($form_input->{ID});
    }
    if ($action eq "ajax_search") {
      my $isodate = $apiis->extdate2iso($form_input->{filter_5});
      $sql  = "SELECT distinct  ";
	  $sql .= "  pj.project_name  as result_col1, ";
	  $sql .= "  cd.ext_code      as result_col2, ";
      $sql .= "  tf.ext_organism  as result_col3, ";
      $sql .= "  sp.ext_sample    as result_col4, ";
      $sql .= "  sp.production_dt as result_col5_dt, ";
      $sql .= "  sp.guid          as guid ";
      $sql .= "FROM sample_collection sc ";
	  $sql .= "  LEFT JOIN samples as sp ON (sp.sample_id = sc.sample_id) ";
   	  $sql .= "  LEFT JOIN codes as cd ON (sp.db_material_type = cd.db_code) ";
   	  $sql .= "  LEFT JOIN transfer as tf ON (sp.db_organism = tf.db_organism) ";
   	  $sql .= "  LEFT JOIN projects as pj ON (sc.project_id = pj.project_id) ";
 	  $sql .= "WHERE ";
      $sql .= sprintf "  sc.project_id::text         like %s ",
              $apiis->DataBase->dbh->quote($form_input->{filter_1_loop});
	  $sql .= sprintf "  AND cd.ext_code         like %s ",
              $apiis->DataBase->dbh->quote($form_input->{filter_2_loop});
      $sql .= sprintf "  AND tf.ext_organism     like %s ",
              $apiis->DataBase->dbh->quote($form_input->{filter_3}.'%');
	  $sql .= sprintf "  AND sp.ext_sample       like %s ",
              $apiis->DataBase->dbh->quote($form_input->{filter_4}.'%');
	  $sql .= sprintf "  AND sp.production_dt::text    like '$isodate%' "
	  if ($form_input->{filter_5} ne '');
      $sql .= "ORDER BY $form_input->{'order_by'} $form_input->{'asc_desc'}";
    }
  }
  elsif ($layout eq "dna"){
	if ($action eq "ajax_fill"){
	  my $ds_guid = $apiis->DataBase->dbh->quote($form_input->{ID});
	  if (exist_validation($ds_guid) eq YES){
		$sql  = "SELECT distinct ";
		$sql .= "  ds.guid              as dna_guid, ";
		$sql .= "  de.guid              as de_guid, ";
		$sql .= "  va.guid              as va_guid, ";
		$sql .= "  gi.guid              as gi_guid, ";
		$sql .= "  pj.project_id        as project_name_loop_div, ";
		$sql .= "  os.ext_sample        as ext_sample, ";
		$sql .= "  ds.ext_sample        as ext_dna, ";
		$sql .= "  ds.db_vessel_type    as db_vessel_type_loop_div, ";
		$sql .= "  ds.production_dt     as production_dt, ";
		$sql .= "  ds.protocol_id       as protocol_name_loop_div, ";
		$sql .= "  ds.storage_dt        as storage_dt, ";
		$sql .= "  ss.no_units          as dna_volume, ";
		$sql .= "  de.dna_concentration as dna_concentration, ";
		$sql .= "  de.dna_purity        as dna_purity, ";
		$sql .= "  ds.db_user           as db_user_loop_div, ";
		$sql .= "  ds.comment           as comment, ";
		$sql .= "  va.protocol_id       as dna_checking_protocol_loop_div, ";
		$sql .= "  va.gel_id            as gel_id, ";
		$sql .= "  gi.lane              as lane, ";
		$sql .= "  va.comment           as gel_comment, ";
		$sql .= "  s1.storage_id        as sp1_0_loop_div, ";
		$sql .= "  s2.storage_id        as sp1_1_loop_div, ";
		$sql .= "  s3.storage_id        as sp1_2_loop_div, ";
		$sql .= "  s4.storage_id        as sp1_3_loop_div, ";
		$sql .= "  s5.storage_name      as sp1_4 ";
		$sql .= "FROM  samples ds, projects pj, dna_extraction de ";
		$sql .= "  LEFT JOIN samples as os ON (de.sample_id = os.sample_id) ";
		$sql .= "      ,storage_samples ss, storage s5  ";
		$sql .= "  INNER JOIN storage as s4 ON (s5.parent_id = s4.storage_id) ";
		$sql .= "  INNER JOIN storage as s3 ON (s4.parent_id = s3.storage_id) ";
		$sql .= "  INNER JOIN storage as s2 ON (s3.parent_id = s2.storage_id) ";
		$sql .= "  INNER JOIN storage as s1 ON (s2.parent_id = s1.storage_id) ";
		$sql .= "  , validation va ";
		$sql .= "  INNER JOIN gel_images as gi ON (va.gel_id = gi.gel_id) ";
		$sql .= "WHERE pj.project_id    = de.project_id ";
		$sql .= "  AND de.dna_id        = ds.sample_id ";
		$sql .= "  AND ds.sample_id     = ss.sample_id ";
		$sql .= "  AND ss.storage_id    = s5.storage_id ";
		$sql .= "  AND de.validation_id = va.validation_id ";
		$sql .= "  AND gi.sample_id     = de.sample_id ";
		$sql .= sprintf "  AND ds.guid::text  = %s", $ds_guid;
	  }
	  else{
		$sql  = "SELECT distinct ";
		$sql .= "  ds.guid              as dna_guid, ";
		$sql .= "  de.guid              as de_guid, ";
		$sql .= "  pj.project_id        as project_name_loop_div, ";
		$sql .= "  os.ext_sample        as ext_sample, ";
		$sql .= "  ds.ext_sample        as ext_dna, ";
		$sql .= "  ds.db_vessel_type    as db_vessel_type_loop_div, ";
		$sql .= "  ds.production_dt     as production_dt, ";
		$sql .= "  ds.protocol_id       as protocol_name_loop_div, ";
		$sql .= "  ds.storage_dt        as storage_dt, ";
		$sql .= "  ss.no_units          as dna_volume, ";
		$sql .= "  de.dna_concentration as dna_concentration, ";
		$sql .= "  de.dna_purity        as dna_purity, ";
		$sql .= "  ds.db_user           as db_user_loop_div, ";
		$sql .= "  ds.comment           as comment, ";
		$sql .= "  s1.storage_id        as sp1_0_loop_div, ";
		$sql .= "  s2.storage_id        as sp1_1_loop_div, ";
		$sql .= "  s3.storage_id        as sp1_2_loop_div, ";
		$sql .= "  s4.storage_id        as sp1_3_loop_div, ";
		$sql .= "  s5.storage_name      as sp1_4 ";
		$sql .= "FROM  samples ds,projects pj,storage_samples ss, storage s5  ";
		$sql .= "  INNER JOIN storage as s4 ON (s5.parent_id = s4.storage_id) ";
		$sql .= "  INNER JOIN storage as s3 ON (s4.parent_id = s3.storage_id) ";
		$sql .= "  INNER JOIN storage as s2 ON (s3.parent_id = s2.storage_id) ";
		$sql .= "  INNER JOIN storage as s1 ON (s2.parent_id = s1.storage_id),";
		$sql .= "  dna_extraction de ";
		$sql .= "  LEFT JOIN samples as os ON (de.sample_id = os.sample_id) ";
		$sql .= "WHERE pj.project_id  = de.project_id ";
		$sql .= "  AND de.dna_id      = ds.sample_id ";
		$sql .= "  AND ds.sample_id   = ss.sample_id ";
		$sql .= "  AND ss.storage_id  = s5.storage_id ";
		$sql .= sprintf "  AND ds.guid::text = %s", $ds_guid;
	  }
	}
	if ($action eq "ajax_search") {
		my $isodate = $apiis->extdate2iso($form_input->{filter_4_dt});
		$sql  = "SELECT distinct  ";
		$sql .= "  p.project_name   as result_col1, ";
		$sql .= "  s2.ext_sample    as result_col2, ";
		$sql .= "  s1.ext_sample    as result_col3, ";
		$sql .= "  s1.production_dt as result_col4_dt, ";
		$sql .= "  s1.guid          as guid ";
		$sql .= "FROM samples s1, codes c, projects p, dna_extraction d  ";
		$sql .= "  INNER JOIN samples as s2 ON (d.sample_id = s2.sample_id) ";
		$sql .= "WHERE s1.db_material_type = c.db_code ";
		$sql .= "AND s1.sample_id  = d.dna_id ";
		$sql .= "AND d.project_id = p.project_id ";
		$sql .= "AND c.ext_code  = 'DNA' ";
		$sql .= sprintf "  AND p.project_id::text  like %s ",
				$apiis->DataBase->dbh->quote($form_input->{filter_1_loop});
		$sql .= sprintf "  AND s2.ext_sample like %s ",
				$apiis->DataBase->dbh->quote($form_input->{filter_2}.'%');
		$sql .= sprintf "  AND s1.ext_sample like %s ",
				$apiis->DataBase->dbh->quote($form_input->{filter_3}.'%');
		$sql .= sprintf "  AND s1.production_dt::text    like '$isodate%' "
		if ($form_input->{filter_4_dt} ne '');
		$sql .= "ORDER BY $form_input->{'order_by'} $form_input->{'asc_desc'}";
	}
  }
  elsif ($layout eq "pcr"){
    if ($action eq "ajax_fill") {
      $sql  = "SELECT DISTINCT";
      $sql .= "  pa.guid              as pa_guid, ";
      $sql .= "  va.guid              as va_guid, ";
      $sql .= "  pj.project_id        as project_name_loop_div, ";
      $sql .= "  pa.ext_amplification as pcr_amplification_id, ";
      $sql .= "  pa.protocol_id       as pcr_amplification_protocol_loop_div, ";
      $sql .= "  mk.marker_id         as marker_loop_div, ";
      $sql .= "  pa.comment           as pcr_comment, ";
      $sql .= "  va.protocol_id       as pcr_validation_protocol_loop_div, ";
      $sql .= "  va.comment           as gel_comment, ";
      $sql .= "  va.gel_id            as gel_id, ";
      $sql .= "  CASE WHEN va.gel_id>0 THEN 'Download' ";
      $sql .= "       ELSE '' ";
      $sql .= "  END as download_file ";
      $sql .= "FROM  projects pj, pcr_amplification pa, amplified_samples am ";
      $sql .= "  LEFT JOIN validation va ON (va.validation_id = am.validation_id) ";
      $sql .= ", pcr_markers pm, markers mk, samples sp ";
      $sql .= "WHERE pj.project_id = pa.project_id ";
      $sql .= "  AND pa.amplification_id = am.amplification_id ";
      $sql .= "  AND pa.amplification_id = pm.amplification_id ";
      $sql .= "  AND pm.marker_id = mk.marker_id ";
      $sql .= "  AND am.dna_id = sp.sample_id ";
      $sql .= "  AND am.guid::text ='$form_input->{ID}'";
    }
    if ($action eq "ajax_search") {
      $sql  = "SELECT DISTINCT ";
      $sql .= "  am.guid                as guid, ";
      $sql .= "  pj.project_name        as result_col1, ";
      $sql .= "  pa.ext_amplification   as result_col2, ";
      $sql .= "  mk.marker_name         as result_col3, ";
      $sql .= "  sp.ext_sample          as result_col4, ";
      $sql .= "  CASE WHEN va.gel_id>0 THEN 'Yes' ";
      $sql .= "       ELSE 'No' ";
      $sql .= "  END as result_col5 ";
      $sql .= "FROM  projects pj, pcr_amplification pa, amplified_samples am ";
      $sql .= "  LEFT JOIN validation va ON (va.validation_id = am.validation_id) ";
      $sql .= ", pcr_markers pm, markers mk, samples sp ";
      $sql .= "WHERE pj.project_id = pa.project_id ";
      $sql .= "  AND pa.amplification_id = am.amplification_id ";
      $sql .= "  AND pa.amplification_id = pm.amplification_id ";
      $sql .= "  AND pm.marker_id = mk.marker_id ";
      $sql .= "  AND am.dna_id = sp.sample_id ";
      $sql .= "  AND pj.project_id::text  like '%".$form_input->{filter_1_loop}."' ";
      $sql .= "  AND pa.amplification_id::text  like '%".$form_input->{filter_2_loop}."' ";
      $sql .= "  AND mk.marker_id::text  like '%".$form_input->{filter_3_loop}."' ";
      $sql .= "  AND sp.ext_sample  like '".$form_input->{filter_4}."%' ";
      $sql .= "ORDER BY $form_input->{'order_by'} $form_input->{'asc_desc'}";
    }
  }
  elsif ($layout eq "electrophoresis"){
    if ($action eq "ajax_fill") {
      $sql  = "SELECT ";
      $sql .= "  el.guid                as el_guid, ";
	  $sql .= "  an.guid                as an_guid, ";
      $sql .= "  el.project_id          as project_name_loop_div, ";
      $sql .= "  el.ext_electrophoresis as ext_electrophoresis, ";
      $sql .= "  an.marker_id          as marker_loop_div, ";
      $sql .= "  el.protocol_id         as protocol_loop_div, ";
      $sql .= "  el.db_software         as software_loop_div, ";
      $sql .= "  el.purpose             as electrophoresis_purpose_loop_div, ";
      $sql .= "  el.db_method           as electrophoresis_method_loop_div, ";
      $sql .= "  el.db_user             as user_loop_div, ";
      $sql .= "  el.electrophoresis_dt  as electrophoresis_dt, ";
      $sql .= "  el.comment             as comment, ";
      $sql .= "  el.product_id       as product_id, ";
      $sql .= "  CASE WHEN el.product_id>0 THEN 'Download' ";
      $sql .= "       ELSE '' ";
      $sql .= "  END as download_file ";
      $sql .= "FROM electrophoresises el ";
      $sql .= "  LEFT JOIN analyzed_samples an ON (an.electrophoresis_id = el.electrophoresis_id) ";
      $sql .= "WHERE el.guid::text='$form_input->{ID}'";
    }
    if ($action eq "ajax_search") {
	  my $isodate = $apiis->extdate2iso($form_input->{filter_4_dt});
      $sql  = "SELECT DISTINCT ";
      $sql .= "  e.guid                as guid, ";
      $sql .= "  p.project_name        as result_col1, ";
      $sql .= "  e.ext_electrophoresis as result_col2, ";
      $sql .= "  m.marker_name         as result_col3, ";
      $sql .= "  e.electrophoresis_dt  as result_col4_dt, ";
      $sql .= "  CASE WHEN e.product_id>0 THEN 'Yes' ";
      $sql .= "       ELSE 'No' ";
      $sql .= "  END as result_col5 ";
      $sql .= "FROM  projects p, electrophoresises e, analyzed_samples an ";
#       $sql .= "  LEFT JOIN analyzed_samples an ON (an.electrophoresis_id = e.electrophoresis_id) ";
      $sql .= ", markers m ";
      $sql .= "WHERE p.project_id = e.project_id ";
	  $sql .= "  AND an.electrophoresis_id = e.electrophoresis_id ";
      $sql .= "  AND an.marker_id = m.marker_id ";
      $sql .= "  AND e.project_id::text  like '%".$form_input->{filter_1_loop}."' ";
      $sql .= "  AND e.electrophoresis_id::text  like '%".$form_input->{filter_2_loop}."' ";
      $sql .= "  AND m.marker_id::text  like '%".$form_input->{filter_3_loop}."' ";
	  $sql .= sprintf "  AND e.electrophoresis_dt::text  like '$isodate%' "
 	  if ($form_input->{filter_4_dt} ne '');
      $sql .= "ORDER BY $form_input->{'order_by'} $form_input->{'asc_desc'}";
    }
  }
  elsif ($layout eq "microsatellites"){
    if ($action eq "ajax_fill") {
      $sql  = "SELECT ";
      $sql .= "  ms.guid                as guid, ";
      $sql .= "  pj.project_name        as project_name, ";
      $sql .= "  mk.marker_name         as marker_name, ";
      $sql .= "  pa.ext_amplification   as pcr_id, ";
      $sql .= "  el.ext_electrophoresis as electrophoresis_id, ";
      $sql .= "  os.ext_sample          as sample_id, ";
      $sql .= "  ds.ext_sample          as dna_id, ";
      $sql .= "  ms.allele1             as allele1, ";
      $sql .= "  ms.allele2             as allele2, ";
      $sql .= "  ms.comment             as comment ";
      $sql .= "FROM projects pj, markers mk, microsatellites ms ";
      $sql .= "  LEFT JOIN samples ds           ON (ms.dna_id=ds.sample_id) ";
      $sql .= "  LEFT JOIN analyzed_samples  an ON (ms.dna_id=an.dna_id and ms.marker_id=an.marker_id) ";
      $sql .= "  LEFT JOIN electrophoresises el ON (el.electrophoresis_id=an.electrophoresis_id) ";
      $sql .= "  LEFT JOIN amplified_samples am ON (ms.dna_id=am.dna_id) ";
      $sql .= "  LEFT JOIN pcr_amplification pa ON (am.amplification_id=pa.amplification_id) ";
      $sql .= "  LEFT JOIN dna_extraction de    ON (de.dna_id=ms.dna_id) ";
      $sql .= "  LEFT JOIN samples os    		ON (de.sample_id=os.sample_id) ";
      $sql .= "WHERE pj.project_id=ms.project_id ";
      $sql .= "  AND ms.marker_id=mk.marker_id ";
      $sql .= sprintf" AND ms.guid::text= %s",
              $apiis->DataBase->dbh->quote($form_input->{ID});
    }
    if ($action eq "ajax_search_microsatellites") {
      $sql  = "SELECT ";
      $sql .= "  ds.ext_sample         as result_col1, ";
      $sql .= "  ms.allele1            as result_col2, ";
      $sql .= "  ms.allele2            as result_col3, ";
      $sql .= "  mk.marker_name        as result_col4, ";
      $sql .= "  pa.ext_amplification  as result_col5, ";
      $sql .= "  el.ext_electrophoresis as result_col6, ";
      $sql .= "  ms.guid               as guid ";
      $sql .= "FROM projects pj, markers mk, microsatellites ms ";
      $sql .= "  LEFT JOIN samples ds           ON (ms.dna_id=ds.sample_id) ";
      $sql .= "  LEFT JOIN analyzed_samples  an ON (ms.dna_id=an.dna_id and ms.marker_id=an.marker_id) ";
      $sql .= "  LEFT JOIN electrophoresises el ON (el.electrophoresis_id=an.electrophoresis_id) ";
      $sql .= "  LEFT JOIN amplified_samples am ON (ms.dna_id=am.dna_id) ";
      $sql .= "  LEFT JOIN pcr_amplification pa ON (am.amplification_id=pa.amplification_id) ";
      $sql .= "  LEFT JOIN dna_extraction de    ON (de.dna_id=ms.dna_id) ";
      $sql .= "  LEFT JOIN samples os    		ON (de.sample_id=os.sample_id) ";
      $sql .= "WHERE pj.project_id=ms.project_id ";
      $sql .= "  AND ms.marker_id=mk.marker_id ";
      $sql .= sprintf " AND os.ext_sample like %s ",
              $apiis->DataBase->dbh->quote($form_input->{filter_1}.'%');
      $sql .= sprintf " AND ds.ext_sample like %s ",
              $apiis->DataBase->dbh->quote($form_input->{filter_2}.'%');
      $sql .= "AND pj.project_id= $form_input->{filter_3_loop} ";
      $sql .= sprintf " AND mk.marker_id::text like %s ",
              $apiis->DataBase->dbh->quote($form_input->{filter_4_loop}.'%');
      $sql .= "ORDER BY $form_input->{'order_by'} $form_input->{'asc_desc'}";
    }
  }
  elsif ($layout eq "sequences"){
    if ($action eq "ajax_fill") {
      $sql  = "SELECT ";
      $sql .= "  sq.guid                as guid, ";
      $sql .= "  pj.project_name        as project_name, ";
      $sql .= "  mk.marker_name         as marker_name, ";
      $sql .= "  pa.ext_amplification   as pcr_id, ";
      $sql .= "  el.ext_electrophoresis as electrophoresis_id, ";
      $sql .= "  os.ext_sample          as sample_id, ";
      $sql .= "  ds.ext_sample          as dna_id, ";
      $sql .= "  sq.sequence            as sequence, ";
      $sql .= "  sq.comment             as comment ";
      $sql .= "FROM projects pj, markers mk, sequences sq ";
      $sql .= "  LEFT JOIN samples ds           ON (sq.dna_id=ds.sample_id) ";
      $sql .= "  LEFT JOIN analyzed_samples  an ON (sq.dna_id=an.dna_id and sq.marker_id=an.marker_id) ";
      $sql .= "  LEFT JOIN electrophoresises el ON (el.electrophoresis_id=an.electrophoresis_id) ";
      $sql .= "  LEFT JOIN amplified_samples am ON (sq.dna_id=am.dna_id) ";
      $sql .= "  LEFT JOIN pcr_amplification pa ON (am.amplification_id=pa.amplification_id) ";
      $sql .= "  LEFT JOIN dna_extraction de    ON (de.dna_id=sq.dna_id) ";
      $sql .= "  LEFT JOIN samples os    		ON (de.sample_id=os.sample_id) ";
      $sql .= "WHERE pj.project_id=sq.project_id ";
      $sql .= "  AND sq.marker_id=mk.marker_id ";
      $sql .= sprintf" AND sq.guid::text= %s", $apiis->DataBase->dbh->quote($form_input->{ID});

    }
    if ($action eq "ajax_search_sequences") {
      $sql  = "SELECT ";
      $sql .= "  ds.ext_sample          as result_col1, ";
      $sql .= "  mk.marker_name         as result_col3, ";
      $sql .= "  pa.ext_amplification   as result_col4, ";
      $sql .= "  el.ext_electrophoresis as result_col5, ";
      $sql .= "  sq.guid                as guid ";
      $sql .= "FROM projects pj, markers mk, sequences sq ";
      $sql .= "  LEFT JOIN samples ds           ON (sq.dna_id=ds.sample_id) ";
      $sql .= "  LEFT JOIN analyzed_samples  an ON (sq.dna_id=an.dna_id and sq.marker_id=an.marker_id) ";
      $sql .= "  LEFT JOIN electrophoresises el ON (el.electrophoresis_id=an.electrophoresis_id) ";
      $sql .= "  LEFT JOIN amplified_samples am ON (sq.dna_id=am.dna_id) ";
      $sql .= "  LEFT JOIN pcr_amplification pa ON (am.amplification_id=pa.amplification_id) ";
      $sql .= "  LEFT JOIN dna_extraction de    ON (de.dna_id=sq.dna_id) ";
      $sql .= "  LEFT JOIN samples os    		ON (de.sample_id=os.sample_id) ";
      $sql .= "WHERE pj.project_id=sq.project_id ";
      $sql .= "  AND sq.marker_id=mk.marker_id ";
      $sql .= sprintf " AND os.ext_sample like %s ",
              $apiis->DataBase->dbh->quote($form_input->{filter_1}.'%');
      $sql .= sprintf " AND ds.ext_sample like %s ",
              $apiis->DataBase->dbh->quote($form_input->{filter_2}.'%');
      $sql .= "AND pj.project_id = $form_input->{filter_3_loop} ";
      $sql .= sprintf " AND mk.marker_id::text like %s ",
              $apiis->DataBase->dbh->quote($form_input->{filter_4_loop}.'%');
      $sql .= "ORDER BY $form_input->{'order_by'} $form_input->{'asc_desc'}";     
    }
  }
  elsif ($layout eq "specified_contact"){
    if ($action eq "ajax_search"){
      my $f1 = $apiis->DataBase->dbh->quote("USERS");
      my $f2 = $apiis->DataBase->dbh->quote($form_input->{filter_2}."%");
      my $f3 = $apiis->DataBase->dbh->quote($form_input->{filter_3}."%");
      my $f4 = $apiis->DataBase->dbh->quote($form_input->{filter_4}."%");
      my $f5 = $apiis->DataBase->dbh->quote($form_input->{filter_5}."%");
      $sql  = "SELECT ";
      $sql .= "  u.ext_id      as ext_id, ";
      $sql .= "  c.first_name  as first_name, ";
      $sql .= "  c.second_name  as last_name, ";
      $sql .= "  c.email       as email, ";
      $sql .= "  u.guid        as guid "; 
      $sql .= "FROM unit u LEFT JOIN contacts c ON u.db_contact=c.db_contact ";
      $sql .= "WHERE ext_unit = " . $f1;
      $sql .= "  AND ext_id::text like " . $f2;
      $sql .= "  AND COALESCE(first_name,'') like " . $f3;
      $sql .= "  AND COALESCE(second_name,'') like " . $f4;
      $sql .= "  AND COALESCE(email,'') like ". $f5;
      $sql .= "ORDER BY $form_input->{'order_by'} ";
      $sql .= " $form_input->{'asc_desc'}";
    }
  }
  elsif ($layout eq "analysis_by_organism"){
    if ($action eq "ajax_search"){
      my $f1 = $apiis->DataBase->dbh->quote($form_input->{filter_1_loop});
      my $f2 = $apiis->DataBase->dbh->quote($form_input->{filter_2}."%");
      $sql  = "SELECT ";
      $sql .= "  pj.project_id   as project_id, ";
      $sql .= "  pj.project_name as project_name, ";
      $sql .= "  sp.ext_sample   as sample_id, ";
      $sql .= "  tf.ext_organism as organism_id, ";
      $sql .= "  cd1.ext_code    as species, ";
      $sql .= "  og.guid         as guid "; 
      $sql .= "FROM projects pj,sample_collection sc,samples sp,organisms og ";
      $sql .= "  left join codes cd1 on (og.db_species=cd1.db_code), "; 
      $sql .= "  transfer tf ";
      $sql .= "WHERE pj.project_id = sc.project_id ";
      $sql .= "  AND sc.sample_id = sp.sample_id ";
      $sql .= "  AND sp.db_organism = og.db_organism ";
      $sql .= "  AND og.db_organism = tf.db_organism ";
      $sql .= "  AND pj.project_id::text = " . $f1;
      $sql .= "  AND COALESCE(tf.ext_organism,'') like ". $f2;
      $sql .= "ORDER BY $form_input->{'order_by'} ";
      $sql .= " $form_input->{'asc_desc'}";
    }
  }
  return $sql;
}

=head2 get_input_names - Get an array of identifications of data fields

When the user clicks on one of action icons (view, update, delete), the system
must know exactly what data fields to feed data. This subroutine returns an array
of input identifications. These identifications might help Ajax functions find 
and fill data correctly.

=over 4

=item B<input>:

    1. A layout name

=item B<output>:

    1. A reference to an array of data identifications

=back

=cut

sub get_input_names {
    $layout_name = shift;
    my @form_fields   = ( );
    if    ( $layout_name eq "codes" ){
        @form_fields = qw(
        	guid
            classname
            ext_code
            short_name
            long_name
            opening_dt
            closing_dt
            comment
        );
    }
    elsif ( $layout_name eq "protocols" ){
        @form_fields = qw(
        	guid
        	protocol_id
            db_protocol_type
            protocol_name
            comment                    
         );
    }
    elsif ( $layout_name eq "contacts" ){
        @form_fields = qw(
            unit__guid
            contacts__guid
            ext_unit
            ext_id
            phone
            phone2
            fax
            email
            account
            bank_name
            bank_zip
            title
            salutation
            birth_dt
            first_name
            second_name
            third_name
            db_language_loop_div
            street
            town
            zip
            db_country_loop_div
	    	label
	    	comment
        );
    }
    elsif ( $layout_name eq "protocols" ){
        @form_fields = qw(
        	guid
        	protocol_id
            db_protocol_type
            protocol_name
            comment                    
         );
    }
    elsif ( $layout_name eq "markers" ){
        @form_fields = qw(
        	guid
        	marker_type
        	marker_name
        	db_primer1_loop_div
        	db_primer2_loop_div
            comment                    
         );
    }
    
    elsif ( $layout_name eq "projects" ){
        @form_fields = qw(
            guid
            project_name
            keyword
            begin_dt
            finish_dt
            description
            comment
        );
    }
    
    elsif ( $layout_name eq "organisms" ) {
        @form_fields = qw(
            my_guid
            transfer__guid
            ext_organism
            db_species_loop_div
            db_breed_loop_div
            latitude
            longitude
            db_donor_type_loop_div
            db_sire_loop_div
            db_dam_loop_div
            db_sex_loop_div
            birth_dt
            birth_year
            comment
	    	image_id
	    	organisms_image_div
        );
    }
    elsif ( $layout_name eq "samples" ) {
        @form_fields = qw(
            sample_guid
			project_name_loop_div
			ext_sample
			ext_organism_loop_div
			db_material_type_loop_div
			production_dt
			protocol_name_loop_div
			storage_dt
			db_vessel_type_loop_div
			volume
			db_user_loop_div
			comment
			sp1_0_loop_div
			sp1_1_loop_div
			sp1_2_loop_div
			sp1_3_loop_div
			sp1_4
		);
	}
	elsif ( $layout_name eq "dna" ) {
		@form_fields = qw(
			dna_guid
			de_guid
			va_guid
			gi_guid
			project_name_loop_div
			ext_sample
			ext_dna
			db_vessel_type_loop_div
			production_dt
			protocol_name_loop_div
			storage_dt
			dna_volume
			dna_concentration
			dna_purity
			db_user_loop_div
			comment
			dna_checking_protocol_loop_div
			gel_id
			lane
			gel_comment
			sp1_0_loop_div
			sp1_1_loop_div
			sp1_2_loop_div
			sp1_3_loop_div
			sp1_4
        );
    }
    elsif ( $layout_name eq "electrophoresis" ) {
        @form_fields = qw(
			el_guid
			an_guid
			project_name_loop_div
			ext_electrophoresis
			marker_loop_div
			protocol_loop_div
			software_loop_div
			electrophoresis_purpose_loop_div
			electrophoresis_method_loop_div
			user_loop_div
			electrophoresis_dt
			comment
			product_id
			download_file
        );
    }
    elsif ( $layout_name eq "pcr" ) {
        @form_fields = qw(
			pa_guid
			va_guid
			project_name_loop_div
			pcr_amplification_id
			pcr_amplification_protocol_loop_div
			marker_loop_div
			pcr_comment
			pcr_validation_protocol_loop_div
			gel_comment
			gel_id
			download_file
        );
    }
    elsif ( $layout_name eq "microsatellites" ) {
        @form_fields = qw(
			guid
			project_name
			marker_name
			pcr_id
			electrophoresis_id
			sample_id
			dna_id
			allele1
			allele2
			comment
        );
    }
    elsif ( $layout_name eq "sequences" ) {
        @form_fields = qw(
			guid
			project_name
			marker_name
			pcr_id
			electrophoresis_id
			sample_id
			dna_id
			sequence
			comment
        );
    }
    # reporting
    elsif ( $layout_name eq "specified_contact" ) {
        @form_fields = qw(
			guid
        );
    }
    elsif ( $layout_name eq "samples_by_project" ) {
        @form_fields = qw(
			filter_1_loop
        );
    }
    elsif ( $layout_name eq "dna_by_project" ) {
        @form_fields = qw(
			filter_1_loop
        );
    }
    elsif ( $layout_name eq "analysis_by_organism" ) {
        @form_fields = qw(
			project_id
			organism_id
        );
    }
    elsif ( $layout_name eq "sequences_per_marker" ) {
        @form_fields = qw(
			filter_1_loop
        );
    }
    elsif ( $layout_name eq "microsatellites_per_marker" ) {
        @form_fields = qw(
			filter_1_loop
        );
    }
    elsif ( $layout_name eq "breeds_project" ) {
        @form_fields = qw(
			filter_1_loop
        );
    }

	%params = (
        form_ref      => \@form_fields,
    );
    return \%params;
}

sub exist_validation {
	my $sp_guid = shift;
    my $sql = "";
    $sql  = "SELECT count(*) FROM samples sp ";
    $sql .= " LEFT JOIN dna_extraction de ON(sp.sample_id=de.dna_id) ";
    $sql .= " INNER JOIN validation va ON(de.validation_id = va.validation_id) ";
    $sql .= "WHERE sp.guid::text = ".$sp_guid." ";
    $apiis->log('notice', 'SQL:' . $sql);
    my $sql_ref = $apiis->DataBase->user_sql($sql);
    $apiis->check_status;
    my @data_array = $sql_ref->handle->fetchrow_array;
    my $counter = -1;
    $counter = $data_array[0] if defined $data_array[0];
    $apiis->log('debug', 'Affected rows:' . $counter);
    if ($counter > 0){
    	$apiis->log('notice', 'DNA validation exist');
    	return YES;
    }
    else{
    	return NO;
    }
}

1;

=head1 AUTHORS

Truong Van Chi Cong <cong.chi@fli.bund.de>

=cut
