-- DROP TABLE unit;
CREATE TABLE unit (
   db_unit           int4,       -- Internal sequence for this record
   ext_unit          text,       -- External Name of this unit
   ext_id            text,       -- External ID within this unit
   db_contact        int4,       -- Contacts
   db_member         int4,       -- Member of a hierarchy
   opening_dt        date,       -- Starting date for this unit
   closing_dt        date,       -- Closing date for this unit
   last_change_dt    timestamp,  -- Timestamp of last change
   last_change_user  text,       -- Who did the last change
   dirty             bool,       -- report errors from check_integrity
   chk_lvl           int2,       -- check level
   guid              int4,       -- global identifier
   owner             text,       -- record class
   version           int4,       -- version
   synch             bool        -- is record targeted for synchronization
);

-- DROP INDEX  uidx_pk_unit;
CREATE UNIQUE INDEX uidx_pk_unit ON unit ( ext_unit, ext_id )
WHERE closing_dt is NULL;

-- DROP INDEX uidx_unit_1;
CREATE UNIQUE INDEX uidx_unit_1 ON unit ( db_unit );
-- DROP INDEX uidx_unit_2;
CREATE UNIQUE INDEX uidx_unit_2 ON unit ( ext_unit, ext_id, closing_dt );

-- DROP INDEX uidx_unit_rowid;
CREATE UNIQUE INDEX uidx_unit_rowid ON unit ( guid );

-- DROP SEQUENCE seq_unit__db_unit;
CREATE SEQUENCE seq_unit__db_unit;

-- DROP TABLE locations;
CREATE TABLE locations (
   db_organism       int4,       -- organism number
   db_location       int4,       -- organism location - it can be farm or enclosure or even field
   entry_dt          date,       -- Date when the organism enters this location
   db_entry_action   int4,       -- Action while entering the location
   exit_dt           date,       -- Date when the organism leaves the location
   db_exit_action    int4,       -- Action while leaving the location
   chk_lvl           int2,       -- Check level
   dirty             bool,       -- Report errors from check_integrity
   guid              int8,       -- Global unique identifier
   last_change_dt    timestamp,  -- Timestamp of the last change
   last_change_user  text,       -- Who did the last change
   owner             text,       -- Record class
   synch             bool,       -- Is the record targeted for synchronization
   version           int2        -- Record version
);

-- DROP INDEX uidx_locations_rowid;
CREATE UNIQUE INDEX uidx_locations_rowid ON locations ( guid );

-- DROP TABLE event;
CREATE TABLE event (
   event_id          int4,       -- Internal ID of the event
   db_event_type     int4,       -- Internal ID of the event type
   db_sampler        int4,       -- Internal ID of the tester
   event_dt          date,       -- Test date
   db_location       int4,       -- Internal ID of the location
   last_change_dt    timestamp,  -- Date of last change,  automatic timestamp
   last_change_user  text,       -- User who did the last change
   dirty             bool,       -- report errors from  check_integrity
   chk_lvl           int2,       -- check level
   guid              int4,       -- global  identifier
   owner             text,       -- record class
   version           int4,       -- version
   synch             bool        -- is record targeted for  synchronization
);

-- DROP INDEX uidx_event_1;
CREATE UNIQUE INDEX uidx_event_1 ON event ( event_id, db_event_type, db_location, event_dt );

-- DROP INDEX uidx_event_rowid;
CREATE UNIQUE INDEX uidx_event_rowid ON event ( guid );

-- DROP SEQUENCE seq_event__event_id;
CREATE SEQUENCE seq_event__event_id;

-- DROP TABLE contacts;
CREATE TABLE contacts (
   db_contact        int4,       -- Internal ID of the contact
   title             text,       -- Professional title: Dr. Prof. PhD
   salutation        text,       -- Salutation: Mr. Mrs.
   first_name        text,       -- First name/Department
   second_name       text,       -- Family/Company name
   third_name        text,       -- Additional names
   birth_dt          date,       -- Birth date
   db_language       int4,       -- Preferred language
   street            text,       -- Name of street + Nr.
   zip               text,       -- Postal zip code
   town              text,       -- Town/City/village
   db_country        int4,       -- Country code
   label             text,       -- Formatted adress for label printing
   phone             text,       -- Phone number
   phone2            text,       -- Alternativ phone number
   fax               text,       -- Fax number
   email             text,       -- Email address
   bank_name         text,       -- Bank name
   bank_zip          text,       -- Bank code (IBAN, SWIFT)
   account           text,       -- Bank account number
   opening_dt        date,       -- Date of opening this record
   closing_dt        date,       -- Date of closing this record
   comment           text,       -- Comment
   chk_lvl           int2,       -- Check level
   dirty             bool,       -- Report errors from check_integrity
   guid              int8,       -- Global unique identifier
   last_change_dt    timestamp,  -- Timestamp of the last change
   last_change_user  text,       -- Who did the last change
   owner             text,       -- Record class
   synch             bool,       -- Is the record targeted for synchronization
   version           int2        -- Record version
);

-- DROP INDEX uidx_contacts_1;
CREATE UNIQUE INDEX uidx_contacts_1 ON contacts ( db_contact );

-- DROP INDEX uidx_contacts_rowid;
CREATE UNIQUE INDEX uidx_contacts_rowid ON contacts ( guid );

-- DROP SEQUENCE seq_contacts__db_contact;
CREATE SEQUENCE seq_contacts__db_contact;

-- DROP TABLE inspool;
CREATE TABLE inspool (
   ds                text,       -- datastream (dataset) name
   record_seq        int4,       -- unique ID of record(sequence)
   in_date           date,       -- Time stamp for initial entry
   ext_unit          int4,       -- Reporting Unit
   proc_dt           date,       -- time stamp for processing
   status            text,       -- Status column
   record            text,       -- the data record
   last_change_dt    timestamp,  -- Timestamp of last change
   last_change_user  text,       -- Who did the last change
   dirty             bool,       -- report errors from check_integrity
   chk_lvl           int2,       -- check level
   guid              int4,       -- global identifier
   owner             text,       -- record class
   version           int4,       -- version
   synch             bool        -- is record targeted for synchronization
);

-- DROP INDEX uidx_inspool_1;
CREATE UNIQUE INDEX uidx_inspool_1 ON inspool ( record_seq );

-- DROP INDEX uidx_inspool_rowid;
CREATE UNIQUE INDEX uidx_inspool_rowid ON inspool ( guid );

-- DROP SEQUENCE seq_inspool__record_seq;
CREATE SEQUENCE seq_inspool__record_seq;

-- DROP TABLE inspool_err;
CREATE TABLE inspool_err (
   record_seq        int4,       -- unique ID of record
   err_type          text,       -- Error type ( DB OS DATA...)
   action            text,       -- Error action
   dbtable           text,       -- Error point to table
   dbcol             text,       -- Error point to column (inside table)
   err_source        text,       -- Location where error occurred
   short_msg         text,       -- Error short message
   long_msg          text,       -- Error long message
   ext_col           text,       -- which external cols are involved
   ext_val           text,       -- external (incoming) value
   mod_val           text,       -- modified value
   comp_val          text,       -- compare values (2 in case of la)
   target_col        text,       -- Main/primary column of this record
   ds                text,       -- data stream
   ext_unit          text,       -- external unit
   status            text,       -- Active of historic?
   err_dt            timestamp,  -- timestamp for setting status
   last_change_dt    timestamp,  -- Timestamp of last change
   last_change_user  text,       -- Who did the last change
   dirty             bool,       -- report errors from check_integrity
   chk_lvl           int2,       -- check level
   guid              int4,       -- global identifier
   owner             text,       -- record class
   version           int4,       -- version
   synch             bool        -- is record targeted for synchronization
);

-- DROP INDEX idx_inspool_err_1;
CREATE  INDEX idx_inspool_err_1 ON inspool_err ( record_seq );

-- DROP INDEX uidx_inspool_err_rowid;
CREATE UNIQUE INDEX uidx_inspool_err_rowid ON inspool_err ( guid );

-- DROP TABLE load_stat;
CREATE TABLE load_stat (
   ds                text,       -- Program name
   job_start         timestamp,  -- timestamp start of job
   job_end           timestamp,  -- timestamp end of job
   status            int4,       -- completion code
   rec_tot_no        int2,       -- Number of Records processed
   rec_err_no        int2,       -- Number of erroneous records
   rec_ok_no         int2,       -- Number of correct records - inserted
   last_change_dt    timestamp,  -- Timestamp of last change
   last_change_user  text,       -- Who did the last change
   dirty             bool,       -- report errors from check_integrity
   chk_lvl           int2,       -- check level
   guid              int4,       -- global identifier
   owner             text,       -- record class
   version           int4,       -- version
   synch             bool        -- is record targeted for synchronization
);

-- DROP INDEX uidx_load_stat_rowid;
CREATE UNIQUE INDEX uidx_load_stat_rowid ON load_stat ( guid );

-- DROP TABLE new_pest;
CREATE TABLE new_pest (
   class             text,       -- group/class (organism, perm_u, fix...)
   key               text,       -- db_organism...
   trait             text,       -- which estimator
   estimator         float8,     -- predictor / estimator
   pev               float8,     -- predicted error variance
   last_change_dt    timestamp,  -- Timestamp of last change
   last_change_user  text,       -- Who did the last change
   dirty             bool,       -- report errors from check_integrity
   chk_lvl           int2,       -- check level
   guid              int4,       -- global identifier
   owner             text,       -- record class
   version           int4,       -- version
   synch             bool        -- is record targeted for synchronization
);

-- DROP INDEX uidx_new_pest_1;
CREATE UNIQUE INDEX uidx_new_pest_1 ON new_pest ( class, key, trait );

-- DROP INDEX uidx_new_pest_rowid;
CREATE UNIQUE INDEX uidx_new_pest_rowid ON new_pest ( guid );

-- DROP TABLE sources;
CREATE TABLE sources (
   source            text,       -- source node
   tablename         text,       -- table name
   class             text,       -- owner node
   columnnames       text,       -- columns
   last_change_dt    timestamp,  -- Date of last change, automatic timestamp
   last_change_user  text,       -- User who did the last change
   dirty             bool,       -- report errors from check_integrity
   chk_lvl           int2,       -- check level
   guid              int4,       -- global identifier
   owner             text,       -- record class
   version           int4,       -- version
   synch             bool        -- is record targeted for synchronization
);

-- DROP INDEX uidx_sources_1;
CREATE UNIQUE INDEX uidx_sources_1 ON sources ( guid );

-- DROP INDEX uidx_sources_rowid;
CREATE UNIQUE INDEX uidx_sources_rowid ON sources ( guid );

-- DROP TABLE targets;
CREATE TABLE targets (
   target            text,       -- target node
   tablename         text,       -- table name
   class             text,       -- owner node
   columnnames       text,       -- columns
   last_change_dt    timestamp,  -- Date of last change, automatic timestamp
   last_change_user  text,       -- User who did the last change
   dirty             bool,       -- report errors from check_integrity
   chk_lvl           int2,       -- check level
   guid              int4,       -- global identifier
   owner             text,       -- record class
   version           int4,       -- version
   synch             bool        -- is record targeted for synchronization
);

-- DROP INDEX uidx_targets_1;
CREATE UNIQUE INDEX uidx_targets_1 ON targets ( guid );

-- DROP INDEX uidx_targets_rowid;
CREATE UNIQUE INDEX uidx_targets_rowid ON targets ( guid );

-- DROP TABLE nodes;
CREATE TABLE nodes (
   nodename          text,       -- node name
   address           text,       -- node ip address
   last_change_dt    timestamp,  -- Date of last change, automatic timestamp
   last_change_user  text,       -- User who did the last change
   dirty             bool,       -- report errors from check_integrity
   chk_lvl           int2,       -- check level
   guid              int4,       -- global identifier
   owner             text,       -- record class
   version           int4,       -- version
   synch             bool        -- is record targeted for synchronization
);

-- DROP INDEX uidx_nodes_1;
CREATE UNIQUE INDEX uidx_nodes_1 ON nodes ( guid );

-- DROP INDEX uidx_nodes_rowid;
CREATE UNIQUE INDEX uidx_nodes_rowid ON nodes ( guid );

-- DROP SEQUENCE seq_database__guid;
CREATE SEQUENCE seq_database__guid;

-- DROP TABLE blobs;
CREATE TABLE blobs (
   blob_id           int4,       -- Number of blob
   blob              bytea,      -- binary large objects
   db_mimetype       int4,       -- Mime_type of the file
   last_change_dt    timestamp,  -- Date of last change, automatic timestamp
   last_change_user  text,       -- User who did the last change
   dirty             bool,       -- report errors from check_integrity
   chk_lvl           int2,       -- check level
   guid              int4,       -- global identifier
   owner             text,       -- record class
   version           int4,       -- version
   synch             bool        -- is record targeted for synchronization
);

-- DROP INDEX uidx_blobs_rowid;
CREATE UNIQUE INDEX uidx_blobs_rowid ON blobs ( guid );

-- DROP SEQUENCE seq_blobs__blob_id;
CREATE SEQUENCE seq_blobs__blob_id;

-- DROP TABLE languages;
CREATE TABLE languages (
   lang_id           int4,       -- language id
   iso_lang          text,       -- ISO 639-1
   lang              text,       -- language
   last_change_dt    timestamp,  -- Timestamp of last change
   last_change_user  text,       -- Who did the last change
   creation_dt       timestamp,  -- Timestamp of creation
   creation_user     text,       -- Who did the creation
   end_dt            timestamp,  -- Timestamp of end using
   end_user          text,       -- Who did the end status
   dirty             bool,       -- report errors from check_integrity
   chk_lvl           int2,       -- check level
   guid              int4,       -- global identifier
   owner             text,       -- record class
   version           int4,       -- version
   synch             bool        -- is record targeted for synchronization
);

-- DROP INDEX uidx_languages_1;
CREATE UNIQUE INDEX uidx_languages_1 ON languages ( lang_id );
-- DROP INDEX uidx_languages_2;
CREATE UNIQUE INDEX uidx_languages_2 ON languages ( iso_lang );
-- DROP INDEX uidx_languages_3;
CREATE UNIQUE INDEX uidx_languages_3 ON languages ( guid );

-- DROP INDEX uidx_languages_rowid;
CREATE UNIQUE INDEX uidx_languages_rowid ON languages ( guid );

-- DROP SEQUENCE seq_languages__lang_id;
CREATE SEQUENCE seq_languages__lang_id;

-- DROP TABLE ar_users;
CREATE TABLE ar_users (
   user_id               int4,       -- unique user number - internal sequence
   user_login            text,       -- login name
   user_password         text,       -- user password
   user_language_id      int4,       -- default language of the user
   user_marker           text,       -- private user identifier which is insert into the owner column
   user_disabled         bool,       -- checking if the user can login to the system
   user_status           bool,       -- current login status of the user (is logged or not)
   user_last_login       timestamp,  -- date of the last login to the system
   user_last_activ_time  time,       -- the time which is updated during the user session after each opertion
   user_session_id       text,       -- session_id  for the interface
   last_change_dt        timestamp,  -- Date of last change, automatic timestamp
   last_change_user      text,       -- User who did the last change
   dirty                 bool,       -- report errors from check_integrity
   chk_lvl               int2,       -- check level
   guid                  int4,       -- global identifier
   owner                 text,       -- record class
   version               int4,       -- version
   synch                 bool        -- is record targeted for synchronization
);

-- DROP INDEX uidx_ar_users_1;
CREATE UNIQUE INDEX uidx_ar_users_1 ON ar_users ( user_id );
-- DROP INDEX uidx_ar_users_2;
CREATE UNIQUE INDEX uidx_ar_users_2 ON ar_users ( user_login );
-- DROP INDEX uidx_ar_users_3;
CREATE UNIQUE INDEX uidx_ar_users_3 ON ar_users ( guid );

-- DROP INDEX uidx_ar_users_rowid;
CREATE UNIQUE INDEX uidx_ar_users_rowid ON ar_users ( guid );

-- DROP SEQUENCE seq_ar_users__user_id;
CREATE SEQUENCE seq_ar_users__user_id;

-- DROP TABLE ar_users_data;
CREATE TABLE ar_users_data (
   user_id           int4,       -- Foreign Key to the user table
   user_first_name   text,       -- First name/Department
   user_second_name  text,       -- Family/company name
   user_institution  text,       -- company name
   user_email        text,       -- Email address
   user_country      text,       -- country of the user
   user_street       text,       -- Name of Street + Nr.
   user_town         text,       -- Town name
   user_zip          text,       -- Postal zip code
   user_other_info   text,       -- Comments
   opening_dt        date,       -- Date of opening this record
   closing_dt        date,       -- Date of closing this record
   last_change_dt    timestamp,  -- Timestamp of last change
   last_change_user  text,       -- Who did the last change
   creation_dt       timestamp,  -- Timestamp of creation
   creation_user     text,       -- Who did the creation
   end_dt            timestamp,  -- Timestamp of end using
   end_user          text,       -- Who did the end status
   dirty             bool,       -- report errors from check_integrity
   chk_lvl           int2,       -- check level
   guid              int4,       -- global identifier
   owner             text,       -- record class
   version           int4,       -- version
   synch             bool        -- is record targeted for synchronization
);

-- DROP INDEX uidx_ar_users_data_1;
CREATE UNIQUE INDEX uidx_ar_users_data_1 ON ar_users_data ( user_id );
-- DROP INDEX uidx_ar_users_data_2;
CREATE UNIQUE INDEX uidx_ar_users_data_2 ON ar_users_data ( guid );

-- DROP INDEX uidx_ar_users_data_rowid;
CREATE UNIQUE INDEX uidx_ar_users_data_rowid ON ar_users_data ( guid );

-- DROP TABLE ar_roles;
CREATE TABLE ar_roles (
   role_id           int4,       -- unique role number - internal sequence
   role_name         text,       -- unique name of the role
   role_long_name    text,       -- long role name
   role_type         text,       -- the role type which can be specified as ST (System Task) or DBT (Database Task)
   role_subset       text,       -- Names of the roles which are defined as a subset of role
   role_descr        text,       -- description of the role
   last_change_dt    timestamp,  -- Date of last change, automatic timestamp
   last_change_user  text,       -- User who did the last change
   dirty             bool,       -- report errors from check_integrity
   chk_lvl           int2,       -- check level
   guid              int4,       -- global identifier
   owner             text,       -- record class
   version           int4,       -- version
   synch             bool        -- is record targeted for synchronization
);

-- DROP INDEX uidx_ar_roles_1;
CREATE UNIQUE INDEX uidx_ar_roles_1 ON ar_roles ( role_id );
-- DROP INDEX uidx_ar_roles_2;
CREATE UNIQUE INDEX uidx_ar_roles_2 ON ar_roles ( role_name, role_type );
-- DROP INDEX uidx_ar_roles_3;
CREATE UNIQUE INDEX uidx_ar_roles_3 ON ar_roles ( guid );

-- DROP INDEX uidx_ar_roles_rowid;
CREATE UNIQUE INDEX uidx_ar_roles_rowid ON ar_roles ( guid );

-- DROP SEQUENCE seq_ar_roles__role_id;
CREATE SEQUENCE seq_ar_roles__role_id;

-- DROP TABLE ar_user_roles;
CREATE TABLE ar_user_roles (
   user_id           int4,       -- Foreign Key to the user table
   role_id           int4,       -- Foreign Key to the role table
   last_change_dt    timestamp,  -- Date of last change, automatic timestamp
   last_change_user  text,       -- User who did the last change
   dirty             bool,       -- report errors from check_integrity
   chk_lvl           int2,       -- check level
   guid              int4,       -- global identifier
   owner             text,       -- record class
   version           int4,       -- version
   synch             bool        -- is record targeted for synchronization
);

-- DROP INDEX uidx_ar_user_roles_1;
CREATE UNIQUE INDEX uidx_ar_user_roles_1 ON ar_user_roles ( user_id, role_id );
-- DROP INDEX uidx_ar_user_roles_2;
CREATE UNIQUE INDEX uidx_ar_user_roles_2 ON ar_user_roles ( guid );

-- DROP INDEX uidx_ar_user_roles_rowid;
CREATE UNIQUE INDEX uidx_ar_user_roles_rowid ON ar_user_roles ( guid );

-- DROP TABLE ar_dbtpolicies;
CREATE TABLE ar_dbtpolicies (
   dbtpolicy_id      int4,       -- unique policy number for the database tasks - internal sequence
   action_id         int4,       -- Foreign Key to the codes table - SQL action type
   table_id          int4,       -- Foreign Key to the ar_dbttables
   descriptor_id     int4,       -- Foreign Key to the descriptor table
   last_change_dt    timestamp,  -- Date of last change, automatic timestamp
   last_change_user  text,       -- User who did the last change
   dirty             bool,       -- report errors from check_integrity
   chk_lvl           int2,       -- check level
   guid              int4,       -- global identifier
   owner             text,       -- record class
   version           int4,       -- version
   synch             bool        -- is record targeted for synchronization
);

-- DROP INDEX uidx_ar_dbtpolicies_1;
CREATE UNIQUE INDEX uidx_ar_dbtpolicies_1 ON ar_dbtpolicies ( dbtpolicy_id );
-- DROP INDEX uidx_ar_dbtpolicies_2;
CREATE UNIQUE INDEX uidx_ar_dbtpolicies_2 ON ar_dbtpolicies ( action_id, table_id, descriptor_id );
-- DROP INDEX uidx_ar_dbtpolicies_3;
CREATE UNIQUE INDEX uidx_ar_dbtpolicies_3 ON ar_dbtpolicies ( guid );

-- DROP INDEX uidx_ar_dbtpolicies_rowid;
CREATE UNIQUE INDEX uidx_ar_dbtpolicies_rowid ON ar_dbtpolicies ( guid );

-- DROP SEQUENCE seq_ar_dbtpolicies__dbtpolicy_id;
CREATE SEQUENCE seq_ar_dbtpolicies__dbtpolicy_id;

-- DROP TABLE ar_role_dbtpolicies;
CREATE TABLE ar_role_dbtpolicies (
   role_id           int4,       -- Foreign Key to the role table
   dbtpolicy_id      int4,       -- Foreign Key to the database policies table
   last_change_dt    timestamp,  -- Date of last change, automatic timestamp
   last_change_user  text,       -- User who did the last change
   dirty             bool,       -- report errors from check_integrity
   chk_lvl           int2,       -- check level
   guid              int4,       -- global identifier
   owner             text,       -- record class
   version           int4,       -- version
   synch             bool        -- is record targeted for synchronization
);

-- DROP INDEX uidx_ar_role_dbtpolicies_1;
CREATE UNIQUE INDEX uidx_ar_role_dbtpolicies_1 ON ar_role_dbtpolicies ( role_id, dbtpolicy_id );
-- DROP INDEX uidx_ar_role_dbtpolicies_2;
CREATE UNIQUE INDEX uidx_ar_role_dbtpolicies_2 ON ar_role_dbtpolicies ( guid );

-- DROP INDEX uidx_ar_role_dbtpolicies_rowid;
CREATE UNIQUE INDEX uidx_ar_role_dbtpolicies_rowid ON ar_role_dbtpolicies ( guid );

-- DROP TABLE ar_dbttables;
CREATE TABLE ar_dbttables (
   table_id          int4,       -- unique number - internal sequence
   table_name        text,       -- table name
   table_columns     text,       -- the columns of defined table
   table_desc        text,       -- description of the table
   last_change_dt    timestamp,  -- Date of last change, automatic timestamp
   last_change_user  text,       -- User who did the last change
   dirty             bool,       -- report errors from check_integrity
   chk_lvl           int2,       -- check level
   guid              int4,       -- global identifier
   owner             text,       -- record class
   version           int4,       -- version
   synch             bool        -- is record targeted for synchronization
);

-- DROP INDEX uidx_ar_dbttables_1;
CREATE UNIQUE INDEX uidx_ar_dbttables_1 ON ar_dbttables ( table_id );
-- DROP INDEX uidx_ar_dbttables_2;
CREATE UNIQUE INDEX uidx_ar_dbttables_2 ON ar_dbttables ( table_name, table_columns );
-- DROP INDEX uidx_ar_dbttables_3;
CREATE UNIQUE INDEX uidx_ar_dbttables_3 ON ar_dbttables ( guid );

-- DROP INDEX uidx_ar_dbttables_rowid;
CREATE UNIQUE INDEX uidx_ar_dbttables_rowid ON ar_dbttables ( guid );

-- DROP SEQUENCE seq_ar_dbttabels__table_id;
CREATE SEQUENCE seq_ar_dbttabels__table_id;

-- DROP TABLE ar_dbtdescriptors;
CREATE TABLE ar_dbtdescriptors (
   descriptor_id     int4,       -- unique descriptor number - internal sequence
   descriptor_name   text,       -- descriptor name is a column name which is used as a one of the filter for the access rights
   descriptor_value  text,       -- the values for defined descriptor (column)
   descriptor_desc   text,       -- description of the descriptor
   last_change_dt    timestamp,  -- Date of last change, automatic timestamp
   last_change_user  text,       -- User who did the last change
   dirty             bool,       -- report errors from check_integrity
   chk_lvl           int2,       -- check level
   guid              int4,       -- global identifier
   owner             text,       -- record class
   version           int4,       -- version
   synch             bool        -- is record targeted for synchronization
);

-- DROP INDEX uidx_ar_dbtdescriptors_1;
CREATE UNIQUE INDEX uidx_ar_dbtdescriptors_1 ON ar_dbtdescriptors ( descriptor_id );
-- DROP INDEX uidx_ar_dbtdescriptors_2;
CREATE UNIQUE INDEX uidx_ar_dbtdescriptors_2 ON ar_dbtdescriptors ( descriptor_name, descriptor_value );
-- DROP INDEX uidx_ar_dbtdescriptors_3;
CREATE UNIQUE INDEX uidx_ar_dbtdescriptors_3 ON ar_dbtdescriptors ( guid );

-- DROP INDEX uidx_ar_dbtdescriptors_rowid;
CREATE UNIQUE INDEX uidx_ar_dbtdescriptors_rowid ON ar_dbtdescriptors ( guid );

-- DROP SEQUENCE seq_ar_dbtdescriptor__descriptor_id;
CREATE SEQUENCE seq_ar_dbtdescriptor__descriptor_id;

-- DROP TABLE ar_stpolicies;
CREATE TABLE ar_stpolicies (
   stpolicy_id       int4,       -- unique policy number for the system tasks - internal sequence
   stpolicy_name     text,       -- name of the application or form or report or some other action
   stpolicy_type     text,       -- type of system task policy (www, report, form, action, subroutine)
   stpolicy_desc     text,       -- system policy description
   last_change_dt    timestamp,  -- Date of last change, automatic timestamp
   last_change_user  text,       -- User who did the last change
   dirty             bool,       -- report errors from check_integrity
   chk_lvl           int2,       -- check level
   guid              int4,       -- global identifier
   owner             text,       -- record class
   version           int4,       -- version
   synch             bool        -- is record targeted for synchronization
);

-- DROP INDEX uidx_ar_stpolicies_1;
CREATE UNIQUE INDEX uidx_ar_stpolicies_1 ON ar_stpolicies ( stpolicy_id );
-- DROP INDEX uidx_ar_stpolicies_2;
CREATE UNIQUE INDEX uidx_ar_stpolicies_2 ON ar_stpolicies ( stpolicy_name, stpolicy_type );
-- DROP INDEX uidx_ar_stpolicies_3;
CREATE UNIQUE INDEX uidx_ar_stpolicies_3 ON ar_stpolicies ( guid );

-- DROP INDEX uidx_ar_stpolicies_rowid;
CREATE UNIQUE INDEX uidx_ar_stpolicies_rowid ON ar_stpolicies ( guid );

-- DROP SEQUENCE seq_ar_stpolicies__stpolicy_id;
CREATE SEQUENCE seq_ar_stpolicies__stpolicy_id;

-- DROP TABLE ar_role_stpolicies;
CREATE TABLE ar_role_stpolicies (
   role_id           int4,       -- Foreign Key to the role table
   stpolicy_id       int4,       -- Foreign Key to the system policies table
   last_change_dt    timestamp,  -- Date of last change, automatic timestamp
   last_change_user  text,       -- User who did the last change
   dirty             bool,       -- report errors from check_integrity
   chk_lvl           int2,       -- check level
   guid              int4,       -- global identifier
   owner             text,       -- record class
   version           int4,       -- version
   synch             bool        -- is record targeted for synchronization
);

-- DROP INDEX uidx_ar_role_stpolicies_1;
CREATE UNIQUE INDEX uidx_ar_role_stpolicies_1 ON ar_role_stpolicies ( role_id, stpolicy_id );
-- DROP INDEX uidx_ar_role_stpolicies_2;
CREATE UNIQUE INDEX uidx_ar_role_stpolicies_2 ON ar_role_stpolicies ( guid );

-- DROP INDEX uidx_ar_role_stpolicies_rowid;
CREATE UNIQUE INDEX uidx_ar_role_stpolicies_rowid ON ar_role_stpolicies ( guid );

-- DROP TABLE ar_constraints;
CREATE TABLE ar_constraints (
   cons_id           int4,       -- unique constraints number - internal sequence
   cons_name         text,       -- constraints name
   cons_type         text,       -- constraints type which can be defined as: user-group, group-group, role-group
   cons_desc         text,       -- constraint description
   last_change_dt    timestamp,  -- Date of last change, automatic timestamp
   last_change_user  text,       -- User who did the last change
   dirty             bool,       -- report errors from check_integrity
   chk_lvl           int2,       -- check level
   guid              int4,       -- global identifier
   owner             text,       -- record class
   version           int4,       -- version
   synch             bool        -- is record targeted for synchronization
);

-- DROP INDEX uidx_ar_constraints_1;
CREATE UNIQUE INDEX uidx_ar_constraints_1 ON ar_constraints ( cons_id );
-- DROP INDEX uidx_ar_constraints_2;
CREATE UNIQUE INDEX uidx_ar_constraints_2 ON ar_constraints ( cons_name, cons_type );
-- DROP INDEX uidx_ar_constraints_3;
CREATE UNIQUE INDEX uidx_ar_constraints_3 ON ar_constraints ( guid );

-- DROP INDEX uidx_ar_constraints_rowid;
CREATE UNIQUE INDEX uidx_ar_constraints_rowid ON ar_constraints ( guid );

-- DROP SEQUENCE seq_ar_constraints__cons_id;
CREATE SEQUENCE seq_ar_constraints__cons_id;

-- DROP TABLE ar_role_constraints;
CREATE TABLE ar_role_constraints (
   cons_id           int4,       -- Foreign Key to the constraints table
   first_role_id     int4,       -- Foreign Key to the roles table
   second_role_id    int4,       -- Foreign Key to the roles table
   last_change_dt    timestamp,  -- Date of last change, automatic timestamp
   last_change_user  text,       -- User who did the last change
   dirty             bool,       -- report errors from check_integrity
   chk_lvl           int2,       -- check level
   guid              int4,       -- global identifier
   owner             text,       -- record class
   version           int4,       -- version
   synch             bool        -- is record targeted for synchronization
);

-- DROP INDEX uidx_ar_role_constraints_1;
CREATE UNIQUE INDEX uidx_ar_role_constraints_1 ON ar_role_constraints ( cons_id, first_role_id, second_role_id );
-- DROP INDEX uidx_ar_role_constraints_2;
CREATE UNIQUE INDEX uidx_ar_role_constraints_2 ON ar_role_constraints ( guid );

-- DROP INDEX uidx_ar_role_constraints_rowid;
CREATE UNIQUE INDEX uidx_ar_role_constraints_rowid ON ar_role_constraints ( guid );

-- DROP TABLE codes;
CREATE TABLE codes (
   ext_code          text,       -- The external identification of the code
   class             text,       -- Class/group of codes like SEX, SPECIES, LANGUAGES.
   db_code           int4,       -- The internal identification of the code
   short_name        text,       -- The short name of the code
   long_name         text,       -- The internal identification of the code
   description       text,       -- The arbitrary long code description
   opening_dt        date,       -- Date when this code is defined
   closing_dt        date,       -- Date when this code definition is revoked
   last_change_dt    timestamp,  -- Timestamp of last change
   last_change_user  text,       -- Who did the last change
   dirty             bool,       -- report errors from check_integrity
   chk_lvl           int2,       -- check level
   guid              int4,       -- global identifier
   owner             text,       -- record class
   version           int4,       -- version
   synch             bool        -- is record targeted for synchronization
);

-- DROP INDEX  uidx_pk_codes;
CREATE UNIQUE INDEX uidx_pk_codes ON codes ( class, ext_code )
WHERE closing_dt is NULL;

-- DROP INDEX uidx_codes_1;
CREATE UNIQUE INDEX uidx_codes_1 ON codes ( db_code );
-- DROP INDEX uidx_codes_2;
CREATE UNIQUE INDEX uidx_codes_2 ON codes ( class, ext_code, closing_dt );

-- DROP INDEX uidx_codes_rowid;
CREATE UNIQUE INDEX uidx_codes_rowid ON codes ( guid );

-- DROP SEQUENCE seq_codes__db_code;
CREATE SEQUENCE seq_codes__db_code;

-- DROP TABLE transfer;
CREATE TABLE transfer (
   db_organism       int4,       -- Internal database ID
   ext_organism      text,       -- external ID of the organism
   db_unit           int4,       -- Internal ID of reporting unit
   id_set            int4,       -- Set of categories for the numbering scheme
   opening_dt        date,       -- Date when this channel is opened for new data
   closing_dt        date,       -- Date when this data channel is closed
   last_change_dt    timestamp,  -- Timestamp of last change
   last_change_user  text,       -- Who did the last change
   dirty             bool,       -- report errors from check_integrity
   chk_lvl           int2,       -- check level
   guid              int4,       -- global identifier
   owner             text,       -- record class
   version           int4,       -- version
   synch             bool        -- is record targeted for synchronization
);

-- DROP INDEX  uidx_pk_transfer;
CREATE UNIQUE INDEX uidx_pk_transfer ON transfer ( db_unit, ext_organism )
WHERE closing_dt is NULL;

-- DROP INDEX idx_transfer_1;
CREATE  INDEX idx_transfer_1 ON transfer ( db_organism );
-- DROP INDEX idx_transfer_2;
CREATE  INDEX idx_transfer_2 ON transfer ( ext_organism, db_unit );

-- DROP INDEX uidx_transfer_rowid;
CREATE UNIQUE INDEX uidx_transfer_rowid ON transfer ( guid );

-- DROP SEQUENCE seq_transfer__db_organism;
CREATE SEQUENCE seq_transfer__db_organism;

-- DROP TABLE organisms;
CREATE TABLE organisms (
   db_organism       int4,       -- organism Number
   db_sire           int4,       -- Sire number
   db_dam            int4,       -- Dam number
   db_sex            int4,       -- The sex of the animal
   birth_dt          date,       -- Birthday of the animal
   birth_year        text,       -- Birthyear is used we do not know the birthday exactly
   latitude          float8,     -- S/N latitude - decimal degrees
   longitude         float8,     -- W/S longitude - decimal degrees
   db_donor_type     int4,       -- Type of the donor (animal or plant)
   db_species        int4,       -- Species of the breed
   db_breed          int4,       -- Animal breeds or plant varieties
   db_photo          int4,       -- Picture of the animal or plant
   comment           text,       -- Comments
   la_rep            text,       -- last_action in reproduction
   la_rep_dt         date,       -- last_action in reproduction DATE
   last_change_dt    timestamp,  -- Timestamp of last change
   last_change_user  text,       -- Who did the last change
   dirty             bool,       -- report errors from check_integrity
   chk_lvl           int2,       -- check level
   guid              int4,       -- global identifier
   owner             text,       -- record class
   version           int4,       -- version
   synch             bool,       -- is record targeted for synchronization
   db_hybrid         int4        -- Purebred or hybrid
);

-- DROP INDEX uidx_organisms_1;
CREATE UNIQUE INDEX uidx_organisms_1 ON organisms ( db_organism );

-- DROP INDEX uidx_organisms_rowid;
CREATE UNIQUE INDEX uidx_organisms_rowid ON organisms ( guid );

-- DROP TABLE storage;
CREATE TABLE storage (
   storage_id        int4,       -- ID for the unit place where the material is placed
   storage_name      text,       -- Name of storage place
   parent_id         int4,       -- Parent node in hierarchical data tree (storage_id)
   comment           text,       -- Free text for comments
   chk_lvl           int2,       -- Check level
   dirty             bool,       -- Report errors from check_integrity
   guid              int8,       -- Global unique identifier
   last_change_dt    timestamp,  -- Timestamp of the last change
   last_change_user  text,       -- Who did the last change
   owner             text,       -- Record class
   synch             bool,       -- Is the record targeted for synchronization
   version           int2        -- Record version
);

-- DROP INDEX uidx_storage_1;
CREATE UNIQUE INDEX uidx_storage_1 ON storage ( storage_id );

-- DROP INDEX uidx_storage_rowid;
CREATE UNIQUE INDEX uidx_storage_rowid ON storage ( guid );

-- DROP SEQUENCE seq_storage__storage_id;
CREATE SEQUENCE seq_storage__storage_id;

-- DROP TABLE protocols;
CREATE TABLE protocols (
   protocol_id       int4,       -- Internal ID for BLOB
   protocol_name     text,       -- A short name such as: SC2005, SC2006
   comment           text,       -- Free text for comments
   chk_lvl           int2,       -- Check level
   dirty             bool,       -- Report errors from check_integrity
   guid              int8,       -- Global unique identifier
   last_change_dt    timestamp,  -- Timestamp of the last change
   last_change_user  text,       -- Who did the last change
   owner             text,       -- Record class
   synch             bool,       -- Is the record targeted for synchronization
   version           int2,       -- Record version
   db_protocol_type  int4        -- Type of the protocol
);

-- DROP INDEX uidx_protocols_rowid;
CREATE UNIQUE INDEX uidx_protocols_rowid ON protocols ( guid );

-- DROP TABLE projects;
CREATE TABLE projects (
   chk_lvl           int2,       -- Check level
   dirty             bool,       -- Report errors from check_integrity
   guid              int8,       -- Global unique identifier
   last_change_dt    timestamp,  -- Timestamp of the last change
   last_change_user  text,       -- Who did the last change
   owner             text,       -- Record class
   synch             bool,       -- Is the record targeted for synchronization
   version           int2,       -- Record version
   project_id        int4,       -- Internal identification of the project
   project_name      text,       -- Name of the project (e.g., Chicken Sampling VN08) must be unique (i.e. two different projects cannot keep the same name)
   keyword           text,       -- A word or a phase of words without spaces used in a reference work to link to the project
   begin_dt          date,       -- Date when the project starts (date format: DD-MM-YYY)
   finish_dt         date,       -- Date when the project will finish (date format: DD-MM-YYY)
   description       text,       -- You can write a short description of your project. Other details such as the project coordinator who manages all activities in the project, project funds, .. should be put here.
   comment           text        -- Comments or notes
);

-- DROP INDEX uidx_projects_1;
CREATE UNIQUE INDEX uidx_projects_1 ON projects ( project_id );
-- DROP INDEX uidx_projects_2;
CREATE UNIQUE INDEX uidx_projects_2 ON projects ( project_name );

-- DROP INDEX uidx_projects_rowid;
CREATE UNIQUE INDEX uidx_projects_rowid ON projects ( guid );

-- DROP SEQUENCE seq_projects__project_id;
CREATE SEQUENCE seq_projects__project_id;

-- DROP TABLE samples;
CREATE TABLE samples (
   chk_lvl           int2,       -- Check level
   dirty             bool,       -- Report errors from check_integrity
   guid              int8,       -- Global unique identifier
   last_change_dt    timestamp,  -- Timestamp of the last change
   last_change_user  text,       -- Who did the last change
   owner             text,       -- Record class
   synch             bool,       -- Is the record targeted for synchronization
   version           int2,       -- Record version
   sample_id         int4,       -- Internal Sample ID
   ext_sample        text,       -- External identification of the sample or external number of the vessel
   db_organism       int4,       -- Animal or Plant ID
   db_vessel_type    int4,       -- Type of vessel used to store the samples
   db_material_type  int4,       -- Type of genetic materials or samples
   db_user           int4,       -- User who collected or extracted the sample
   protocol_id       int4,       -- Protocol ID
   production_dt     date,       -- Production date when you collect the samples
   storage_dt        date,       -- Storage date when the samples are sent to the storage places
   comment           text        -- Comments
);

-- DROP INDEX uidx_samples_1;
CREATE UNIQUE INDEX uidx_samples_1 ON samples ( sample_id );
-- DROP INDEX uidx_samples_2;
CREATE UNIQUE INDEX uidx_samples_2 ON samples ( ext_sample );

-- DROP INDEX uidx_samples_rowid;
CREATE UNIQUE INDEX uidx_samples_rowid ON samples ( guid );

-- DROP SEQUENCE seq_samples__sample_id;
CREATE SEQUENCE seq_samples__sample_id;

-- DROP TABLE storage_samples;
CREATE TABLE storage_samples (
   chk_lvl           int2,       -- Check level
   dirty             bool,       -- Report errors from check_integrity
   guid              int8,       -- Global unique identifier
   last_change_dt    timestamp,  -- Timestamp of the last change
   last_change_user  text,       -- Who did the last change
   owner             text,       -- Record class
   synch             bool,       -- Is the record targeted for synchronization
   version           int2,       -- Record version
   storage_id        int4,       -- Storage ID
   sample_id         int4,       -- Sample ID
   no_units          int2        -- For semen: the number of straws (replicates) For liquids: amount in units, like milliliter or grams.  They all have the same sample ID. What the unit is, the user has to define (ml or g or kg). This could be put in the protocol or comment field.
);

-- DROP INDEX uidx_storage_samples_rowid;
CREATE UNIQUE INDEX uidx_storage_samples_rowid ON storage_samples ( guid );

-- DROP TABLE sample_collection;
CREATE TABLE sample_collection (
   chk_lvl           int2,       -- Check level
   dirty             bool,       -- Report errors from check_integrity
   guid              int8,       -- Global unique identifier
   last_change_dt    timestamp,  -- Timestamp of the last change
   last_change_user  text,       -- Who did the last change
   owner             text,       -- Record class
   synch             bool,       -- Is the record targeted for synchronization
   version           int2,       -- Record version
   sample_id         int4,       -- Sample ID
   project_id        int4,       -- Project ID
   reused            text        -- N: New sample, R: Reused sample
);

-- DROP INDEX uidx_sample_collection_rowid;
CREATE UNIQUE INDEX uidx_sample_collection_rowid ON sample_collection ( guid );

-- DROP TABLE dna_extraction;
CREATE TABLE dna_extraction (
   chk_lvl            int2,       -- Check level
   dirty              bool,       -- Report errors from check_integrity
   guid               int8,       -- Global unique identifier
   last_change_dt     timestamp,  -- Timestamp of the last change
   last_change_user   text,       -- Who did the last change
   owner              text,       -- Record class
   synch              bool,       -- Is the record targeted for synchronization
   version            int2,       -- Record version
   dna_id             int4,       -- New ID for DNA
   sample_id          int4,       -- Sample ID which DNA is extracted from
   project_id         int4,       -- Project ID
   validation_id      int4,       -- Validation ID
   dna_concentration  float4,     -- DNA concentration
   dna_purity         float4,     -- DNA purity
   reused             text        -- N: New sample, R: Reused sample
);

-- DROP INDEX uidx_dna_extraction_rowid;
CREATE UNIQUE INDEX uidx_dna_extraction_rowid ON dna_extraction ( guid );

-- DROP TABLE validation;
CREATE TABLE validation (
   validation_id     int4,       -- Internal ID
   gel_id            int4,       -- Gel ID
   protocol_id       int4,       -- Protocol ID
   comment           text,       -- Free text for comments
   chk_lvl           int2,       -- Check level
   dirty             bool,       -- Report errors from check_integrity
   guid              int8,       -- Global unique identifier
   last_change_dt    timestamp,  -- Timestamp of the last change
   last_change_user  text,       -- Who did the last change
   owner             text,       -- Record class
   synch             bool,       -- Is the record targeted for synchronization
   version           int2        -- Record version
);

-- DROP INDEX uidx_validation_rowid;
CREATE UNIQUE INDEX uidx_validation_rowid ON validation ( guid );

-- DROP SEQUENCE seq_validation__validation_id;
CREATE SEQUENCE seq_validation__validation_id;

-- DROP TABLE gel_images;
CREATE TABLE gel_images (
   gel_id            int4,       -- GEL ID
   sample_id         int4,       -- Sample ID
   lane              int2,       -- The name of the lane which is used to find the sample inside the gel image
   valid             text,       -- Y: Valid, N: Invalid
   chk_lvl           int2,       -- Check level
   dirty             bool,       -- Report errors from check_integrity
   guid              int8,       -- Global unique identifier
   last_change_dt    timestamp,  -- Timestamp of the last change
   last_change_user  text,       -- Who did the last change
   owner             text,       -- Record class
   synch             bool,       -- Is the record targeted for synchronization
   version           int2        -- Record version
);

-- DROP INDEX uidx_gel_images_rowid;
CREATE UNIQUE INDEX uidx_gel_images_rowid ON gel_images ( guid );

-- DROP TABLE markers;
CREATE TABLE markers (
   marker_id         int2,       -- Marker ID
   marker_name       text,       -- Marker name
   db_marker_type    int4,       -- Type of marker
   db_primer1        int4,       -- First PCR primer
   db_primer2        int4,       -- Second PCR primer
   comment           text,       -- Free text for comments
   chk_lvl           int2,       -- Check level
   dirty             bool,       -- Report errors from check_integrity
   guid              int8,       -- Global unique identifier
   last_change_dt    timestamp,  -- Timestamp of the last change
   last_change_user  text,       -- Who did the last change
   owner             text,       -- Record class
   synch             bool,       -- Is the record targeted for synchronization
   version           int2        -- Record version
);

-- DROP INDEX uidx_markers_1;
CREATE UNIQUE INDEX uidx_markers_1 ON markers ( marker_id );
-- DROP INDEX uidx_markers_2;
CREATE UNIQUE INDEX uidx_markers_2 ON markers ( marker_name );

-- DROP INDEX uidx_markers_rowid;
CREATE UNIQUE INDEX uidx_markers_rowid ON markers ( guid );

-- DROP SEQUENCE seq_markers__marker_id;
CREATE SEQUENCE seq_markers__marker_id;

-- DROP TABLE pcr_amplification;
CREATE TABLE pcr_amplification (
   amplification_id   int4,       -- Amplification ID
   ext_amplification  text,       -- External identification of PCR amplification
   project_id         int4,       -- Project ID
   protocol_id        int4,       -- Protocol ID
   comment            text,       -- Free text for comments
   chk_lvl            int2,       -- Check level
   dirty              bool,       -- Report errors from check_integrity
   guid               int8,       -- Global unique identifier
   last_change_dt     timestamp,  -- Timestamp of the last change
   last_change_user   text,       -- Who did the last change
   owner              text,       -- Record class
   synch              bool,       -- Is the record targeted for synchronization
   version            int2        -- Record version
);

-- DROP INDEX uidx_pcr_amplification_1;
CREATE UNIQUE INDEX uidx_pcr_amplification_1 ON pcr_amplification ( amplification_id );
-- DROP INDEX uidx_pcr_amplification_2;
CREATE UNIQUE INDEX uidx_pcr_amplification_2 ON pcr_amplification ( ext_amplification );

-- DROP INDEX uidx_pcr_amplification_rowid;
CREATE UNIQUE INDEX uidx_pcr_amplification_rowid ON pcr_amplification ( guid );

-- DROP SEQUENCE seq_pcr_amplification__amplification_id;
CREATE SEQUENCE seq_pcr_amplification__amplification_id;

-- DROP TABLE amplified_samples;
CREATE TABLE amplified_samples (
   amplification_id  int4,       -- Amplification ID
   dna_id            int4,       -- DNA ID
   validation_id     int4,       -- Validation ID
   comment           text,       -- Free text for comments
   chk_lvl           int2,       -- Check level
   dirty             bool,       -- Report errors from check_integrity
   guid              int8,       -- Global unique identifier
   last_change_dt    timestamp,  -- Timestamp of the last change
   last_change_user  text,       -- Who did the last change
   owner             text,       -- Record class
   synch             bool,       -- Is the record targeted for synchronization
   version           int2        -- Record version
);

-- DROP INDEX uidx_amplified_samples_rowid;
CREATE UNIQUE INDEX uidx_amplified_samples_rowid ON amplified_samples ( guid );

-- DROP TABLE pcr_markers;
CREATE TABLE pcr_markers (
   amplification_id  int4,       -- Amplification ID
   marker_id         int4,       -- Marker ID
   chk_lvl           int2,       -- Check level
   dirty             bool,       -- Report errors from check_integrity
   guid              int8,       -- Global unique identifier
   last_change_dt    timestamp,  -- Timestamp of the last change
   last_change_user  text,       -- Who did the last change
   owner             text,       -- Record class
   synch             bool,       -- Is the record targeted for synchronization
   version           int2        -- Record version
);

-- DROP INDEX uidx_pcr_markers_rowid;
CREATE UNIQUE INDEX uidx_pcr_markers_rowid ON pcr_markers ( guid );

-- DROP TABLE electrophoresises;
CREATE TABLE electrophoresises (
   electrophoresis_id   int4,       -- Amplification ID
   ext_electrophoresis  text,       -- External identification of Electrophoresis
   project_id           int4,       -- Project ID
   db_user              int4,       -- User who carried out the electrophoresis
   db_software          int4,       -- Used Software
   product_id           int4,       -- Product ID of the electrophoresis result
   protocol_id          int4,       -- Protocol ID
   electrophoresis_dt   date,       -- Data when running electrophoresis
   db_method            int4,       -- Electrophiresis method
   purpose              int4,       -- Electrophiresis purpose
   comment              text,       -- Free text for comments
   chk_lvl              int2,       -- Check level
   dirty                bool,       -- Report errors from check_integrity
   guid                 int8,       -- Global unique identifier
   last_change_dt       timestamp,  -- Timestamp of the last change
   last_change_user     text,       -- Who did the last change
   owner                text,       -- Record class
   synch                bool,       -- Is the record targeted for synchronization
   version              int2        -- Record version
);

-- DROP INDEX uidx_electrophoresises_rowid;
CREATE UNIQUE INDEX uidx_electrophoresises_rowid ON electrophoresises ( guid );

-- DROP SEQUENCE seq_electrophoresises__electrophoresis_id;
CREATE SEQUENCE seq_electrophoresises__electrophoresis_id;

-- DROP TABLE analyzed_samples;
CREATE TABLE analyzed_samples (
   electrophoresis_id  int4,       -- Amplification ID
   dna_id              int4,       -- DNA ID
   marker_id           int4,       -- Marker ID
   chk_lvl             int2,       -- Check level
   dirty               bool,       -- Report errors from check_integrity
   guid                int8,       -- Global unique identifier
   last_change_dt      timestamp,  -- Timestamp of the last change
   last_change_user    text,       -- Who did the last change
   owner               text,       -- Record class
   synch               bool,       -- Is the record targeted for synchronization
   version             int2        -- Record version
);

-- DROP INDEX uidx_analyzed_samples_rowid;
CREATE UNIQUE INDEX uidx_analyzed_samples_rowid ON analyzed_samples ( guid );

-- DROP TABLE microsatellites;
CREATE TABLE microsatellites (
   microsatellite_id  int4,       -- Amplification ID
   dna_id             int4,       -- DNA ID
   project_id         int4,       -- Project ID
   marker_id          int4,       -- Marker ID
   allele1            int2,       -- Marker ID
   allele2            int2,       -- Marker ID
   comment            text,       -- Free text for comments
   chk_lvl            int2,       -- Check level
   dirty              bool,       -- Report errors from check_integrity
   guid               int8,       -- Global unique identifier
   last_change_dt     timestamp,  -- Timestamp of the last change
   last_change_user   text,       -- Who did the last change
   owner              text,       -- Record class
   synch              bool,       -- Is the record targeted for synchronization
   version            int2        -- Record version
);

-- DROP INDEX uidx_microsatellites_rowid;
CREATE UNIQUE INDEX uidx_microsatellites_rowid ON microsatellites ( guid );

-- DROP SEQUENCE seq_microsatellites__microsatellite_id;
CREATE SEQUENCE seq_microsatellites__microsatellite_id;

-- DROP TABLE sequences;
CREATE TABLE sequences (
   sequence_id       int4,       -- Amplification ID
   dna_id            int4,       -- DNA ID
   project_id        int4,       -- Project ID
   marker_id         int4,       -- Marker ID
   sequence          text,       -- Sequence
   comment           text,       -- Free text for comments
   chk_lvl           int2,       -- Check level
   dirty             bool,       -- Report errors from check_integrity
   guid              int8,       -- Global unique identifier
   last_change_dt    timestamp,  -- Timestamp of the last change
   last_change_user  text,       -- Who did the last change
   owner             text,       -- Record class
   synch             bool,       -- Is the record targeted for synchronization
   version           int2        -- Record version
);

-- DROP INDEX uidx_sequences_rowid;
CREATE UNIQUE INDEX uidx_sequences_rowid ON sequences ( guid );

-- DROP SEQUENCE seq_sequences__sequence_id;
CREATE SEQUENCE seq_sequences__sequence_id;

-- DROP VIEW v_unit;
CREATE VIEW v_unit AS
SELECT a.guid AS v_guid,
       a.db_unit,
       a.ext_unit,
       a.ext_id,
       a.db_contact,
       a.db_member,
       b.ext_unit || ':::' || b.ext_id AS ext_member,
       b.opening_dt,
       b.closing_dt,
       b.last_change_dt,
       b.last_change_user,
       b.dirty,
       b.chk_lvl,
       b.guid,
       b.owner,
       b.version,
       b.synch
FROM unit a LEFT OUTER JOIN unit b ON b.db_member = b.db_unit;

-- DROP VIEW v_locations;
CREATE VIEW v_locations AS
SELECT a.guid AS v_guid,
       a.db_organism,
       c.ext_unit || ':::' || c.ext_id || ':::' || b.ext_organism AS ext_organism,
       a.db_location,
       d.ext_unit || ':::' || d.ext_id AS ext_location,
       a.entry_dt,
       a.db_entry_action,
       e.ext_code AS ext_entry_action,
       a.exit_dt,
       a.db_exit_action,
       f.ext_code AS ext_exit_action,
       a.chk_lvl,
       a.dirty,
       a.guid,
       a.last_change_dt,
       a.last_change_user,
       a.owner,
       a.synch,
       a.version
FROM locations a LEFT OUTER JOIN transfer b ON a.db_organism = b.db_organism
                 LEFT OUTER JOIN unit c ON b.db_unit = c.db_unit
                 LEFT OUTER JOIN unit d ON a.db_location = d.db_unit
                 LEFT OUTER JOIN codes e ON a.db_entry_action = e.db_code
                 LEFT OUTER JOIN codes f ON a.db_exit_action = f.db_code;

-- DROP VIEW v_event;
CREATE VIEW v_event AS
SELECT a.guid AS v_guid,
       a.event_id,
       a.db_event_type,
       b.ext_code AS ext_event_type,
       a.db_sampler,
       c.ext_unit || ':::' || c.ext_id AS ext_sampler,
       a.event_dt,
       a.db_location,
       d.ext_unit || ':::' || d.ext_id AS ext_location,
       a.last_change_dt,
       a.last_change_user,
       a.dirty,
       a.chk_lvl,
       a.guid,
       a.owner,
       a.version,
       a.synch
FROM event a LEFT OUTER JOIN codes b ON a.db_event_type = b.db_code
             LEFT OUTER JOIN unit c ON a.db_sampler = c.db_unit
             LEFT OUTER JOIN unit d ON a.db_location = d.db_unit;

-- DROP VIEW v_contacts;
CREATE VIEW v_contacts AS
SELECT a.guid AS v_guid,
       a.db_contact,
       a.title,
       a.salutation,
       a.first_name,
       a.second_name,
       a.third_name,
       a.birth_dt,
       a.db_language,
       b.ext_code AS ext_language,
       a.street,
       a.zip,
       a.town,
       a.db_country,
       c.ext_code AS ext_country,
       a.label,
       a.phone,
       a.phone2,
       a.fax,
       a.email,
       a.bank_name,
       a.bank_zip,
       a.account,
       a.opening_dt,
       a.closing_dt,
       a.comment,
       a.chk_lvl,
       a.dirty,
       a.guid,
       a.last_change_dt,
       a.last_change_user,
       a.owner,
       a.synch,
       a.version
FROM contacts a LEFT OUTER JOIN codes b ON a.db_language = b.db_code
                LEFT OUTER JOIN codes c ON a.db_country = c.db_code;

-- DROP VIEW v_inspool;
CREATE VIEW v_inspool AS
SELECT a.guid AS v_guid,
       a.ds,
       a.record_seq,
       a.in_date,
       a.ext_unit,
       a.proc_dt,
       a.status,
       a.record,
       a.last_change_dt,
       a.last_change_user,
       a.dirty,
       a.chk_lvl,
       a.guid,
       a.owner,
       a.version,
       a.synch
FROM inspool a;

-- DROP VIEW v_inspool_err;
CREATE VIEW v_inspool_err AS
SELECT a.guid AS v_guid,
       a.record_seq,
       a.err_type,
       a.action,
       a.dbtable,
       a.dbcol,
       a.err_source,
       a.short_msg,
       a.long_msg,
       a.ext_col,
       a.ext_val,
       a.mod_val,
       a.comp_val,
       a.target_col,
       a.ds,
       a.ext_unit,
       a.status,
       a.err_dt,
       a.last_change_dt,
       a.last_change_user,
       a.dirty,
       a.chk_lvl,
       a.guid,
       a.owner,
       a.version,
       a.synch
FROM inspool_err a;

-- DROP VIEW v_load_stat;
CREATE VIEW v_load_stat AS
SELECT a.guid AS v_guid,
       a.ds,
       a.job_start,
       a.job_end,
       a.status,
       a.rec_tot_no,
       a.rec_err_no,
       a.rec_ok_no,
       a.last_change_dt,
       a.last_change_user,
       a.dirty,
       a.chk_lvl,
       a.guid,
       a.owner,
       a.version,
       a.synch
FROM load_stat a;

-- DROP VIEW v_new_pest;
CREATE VIEW v_new_pest AS
SELECT a.guid AS v_guid,
       a.class,
       a.key,
       a.trait,
       a.estimator,
       a.pev,
       a.last_change_dt,
       a.last_change_user,
       a.dirty,
       a.chk_lvl,
       a.guid,
       a.owner,
       a.version,
       a.synch
FROM new_pest a;

-- DROP VIEW v_sources;
CREATE VIEW v_sources AS
SELECT a.guid AS v_guid,
       a.source,
       a.tablename,
       a.class,
       a.columnnames,
       a.last_change_dt,
       a.last_change_user,
       a.dirty,
       a.chk_lvl,
       a.guid,
       a.owner,
       a.version,
       a.synch
FROM sources a;

-- DROP VIEW v_targets;
CREATE VIEW v_targets AS
SELECT a.guid AS v_guid,
       a.target,
       a.tablename,
       a.class,
       a.columnnames,
       a.last_change_dt,
       a.last_change_user,
       a.dirty,
       a.chk_lvl,
       a.guid,
       a.owner,
       a.version,
       a.synch
FROM targets a;

-- DROP VIEW v_nodes;
CREATE VIEW v_nodes AS
SELECT a.guid AS v_guid,
       a.nodename,
       a.address,
       a.last_change_dt,
       a.last_change_user,
       a.dirty,
       a.chk_lvl,
       a.guid,
       a.owner,
       a.version,
       a.synch
FROM nodes a;

-- DROP VIEW v_blobs;
CREATE VIEW v_blobs AS
SELECT a.guid AS v_guid,
       a.blob_id,
       a.blob,
       a.db_mimetype,
       b.ext_code AS ext_mimetype,
       a.last_change_dt,
       a.last_change_user,
       a.dirty,
       a.chk_lvl,
       a.guid,
       a.owner,
       a.version,
       a.synch
FROM blobs a LEFT OUTER JOIN codes b ON a.db_mimetype = b.db_code;

-- DROP VIEW v_languages;
CREATE VIEW v_languages AS
SELECT a.guid AS v_guid,
       a.lang_id,
       a.iso_lang,
       a.lang,
       a.last_change_dt,
       a.last_change_user,
       a.creation_dt,
       a.creation_user,
       a.end_dt,
       a.end_user,
       a.dirty,
       a.chk_lvl,
       a.guid,
       a.owner,
       a.version,
       a.synch
FROM languages a;

-- DROP VIEW v_ar_users;
CREATE VIEW v_ar_users AS
SELECT a.guid AS v_guid,
       a.user_id,
       a.user_login,
       a.user_password,
       a.user_language_id,
       a.user_marker,
       a.user_disabled,
       a.user_status,
       a.user_last_login,
       a.user_last_activ_time,
       a.user_session_id,
       a.last_change_dt,
       a.last_change_user,
       a.dirty,
       a.chk_lvl,
       a.guid,
       a.owner,
       a.version,
       a.synch
FROM ar_users a;

-- DROP VIEW v_ar_users_data;
CREATE VIEW v_ar_users_data AS
SELECT a.guid AS v_guid,
       a.user_id,
       a.user_first_name,
       a.user_second_name,
       a.user_institution,
       a.user_email,
       a.user_country,
       a.user_street,
       a.user_town,
       a.user_zip,
       a.user_other_info,
       a.opening_dt,
       a.closing_dt,
       a.last_change_dt,
       a.last_change_user,
       a.creation_dt,
       a.creation_user,
       a.end_dt,
       a.end_user,
       a.dirty,
       a.chk_lvl,
       a.guid,
       a.owner,
       a.version,
       a.synch
FROM ar_users_data a;

-- DROP VIEW v_ar_roles;
CREATE VIEW v_ar_roles AS
SELECT a.guid AS v_guid,
       a.role_id,
       a.role_name,
       a.role_long_name,
       a.role_type,
       a.role_subset,
       a.role_descr,
       a.last_change_dt,
       a.last_change_user,
       a.dirty,
       a.chk_lvl,
       a.guid,
       a.owner,
       a.version,
       a.synch
FROM ar_roles a;

-- DROP VIEW v_ar_user_roles;
CREATE VIEW v_ar_user_roles AS
SELECT a.guid AS v_guid,
       a.user_id,
       a.role_id,
       a.last_change_dt,
       a.last_change_user,
       a.dirty,
       a.chk_lvl,
       a.guid,
       a.owner,
       a.version,
       a.synch
FROM ar_user_roles a;

-- DROP VIEW v_ar_dbtpolicies;
CREATE VIEW v_ar_dbtpolicies AS
SELECT a.guid AS v_guid,
       a.dbtpolicy_id,
       a.action_id,
       b.ext_code AS ext_action_id,
       a.table_id,
       a.descriptor_id,
       a.last_change_dt,
       a.last_change_user,
       a.dirty,
       a.chk_lvl,
       a.guid,
       a.owner,
       a.version,
       a.synch
FROM ar_dbtpolicies a LEFT OUTER JOIN codes b ON a.action_id = b.db_code;

-- DROP VIEW v_ar_role_dbtpolicies;
CREATE VIEW v_ar_role_dbtpolicies AS
SELECT a.guid AS v_guid,
       a.role_id,
       a.dbtpolicy_id,
       a.last_change_dt,
       a.last_change_user,
       a.dirty,
       a.chk_lvl,
       a.guid,
       a.owner,
       a.version,
       a.synch
FROM ar_role_dbtpolicies a;

-- DROP VIEW v_ar_dbttables;
CREATE VIEW v_ar_dbttables AS
SELECT a.guid AS v_guid,
       a.table_id,
       a.table_name,
       a.table_columns,
       a.table_desc,
       a.last_change_dt,
       a.last_change_user,
       a.dirty,
       a.chk_lvl,
       a.guid,
       a.owner,
       a.version,
       a.synch
FROM ar_dbttables a;

-- DROP VIEW v_ar_dbtdescriptors;
CREATE VIEW v_ar_dbtdescriptors AS
SELECT a.guid AS v_guid,
       a.descriptor_id,
       a.descriptor_name,
       a.descriptor_value,
       a.descriptor_desc,
       a.last_change_dt,
       a.last_change_user,
       a.dirty,
       a.chk_lvl,
       a.guid,
       a.owner,
       a.version,
       a.synch
FROM ar_dbtdescriptors a;

-- DROP VIEW v_ar_stpolicies;
CREATE VIEW v_ar_stpolicies AS
SELECT a.guid AS v_guid,
       a.stpolicy_id,
       a.stpolicy_name,
       a.stpolicy_type,
       a.stpolicy_desc,
       a.last_change_dt,
       a.last_change_user,
       a.dirty,
       a.chk_lvl,
       a.guid,
       a.owner,
       a.version,
       a.synch
FROM ar_stpolicies a;

-- DROP VIEW v_ar_role_stpolicies;
CREATE VIEW v_ar_role_stpolicies AS
SELECT a.guid AS v_guid,
       a.role_id,
       a.stpolicy_id,
       a.last_change_dt,
       a.last_change_user,
       a.dirty,
       a.chk_lvl,
       a.guid,
       a.owner,
       a.version,
       a.synch
FROM ar_role_stpolicies a;

-- DROP VIEW v_ar_constraints;
CREATE VIEW v_ar_constraints AS
SELECT a.guid AS v_guid,
       a.cons_id,
       a.cons_name,
       a.cons_type,
       a.cons_desc,
       a.last_change_dt,
       a.last_change_user,
       a.dirty,
       a.chk_lvl,
       a.guid,
       a.owner,
       a.version,
       a.synch
FROM ar_constraints a;

-- DROP VIEW v_ar_role_constraints;
CREATE VIEW v_ar_role_constraints AS
SELECT a.guid AS v_guid,
       a.cons_id,
       a.first_role_id,
       a.second_role_id,
       a.last_change_dt,
       a.last_change_user,
       a.dirty,
       a.chk_lvl,
       a.guid,
       a.owner,
       a.version,
       a.synch
FROM ar_role_constraints a;

-- DROP VIEW v_codes;
CREATE VIEW v_codes AS
SELECT a.guid AS v_guid,
       a.ext_code,
       a.class,
       a.db_code,
       a.short_name,
       a.long_name,
       a.description,
       a.opening_dt,
       a.closing_dt,
       a.last_change_dt,
       a.last_change_user,
       a.dirty,
       a.chk_lvl,
       a.guid,
       a.owner,
       a.version,
       a.synch
FROM codes a;

-- DROP VIEW v_transfer;
CREATE VIEW v_transfer AS
SELECT a.guid AS v_guid,
       a.db_organism,
       a.ext_organism,
       a.db_unit,
       b.ext_unit || ':::' || b.ext_id AS ext_unit,
       a.id_set,
       c.ext_code AS ext_id_set,
       a.opening_dt,
       a.closing_dt,
       a.last_change_dt,
       a.last_change_user,
       a.dirty,
       a.chk_lvl,
       a.guid,
       a.owner,
       a.version,
       a.synch
FROM transfer a LEFT OUTER JOIN unit b ON a.db_unit = b.db_unit
                LEFT OUTER JOIN codes c ON a.id_set = c.db_code;

-- DROP VIEW v_organisms;
CREATE VIEW v_organisms AS
SELECT a.guid AS v_guid,
       a.db_organism,
       c.ext_unit || ':::' || c.ext_id || ':::' || b.ext_organism AS ext_organism,
       a.db_sire,
       e.ext_unit || ':::' || e.ext_id || ':::' || d.ext_organism AS ext_sire,
       a.db_dam,
       g.ext_unit || ':::' || g.ext_id || ':::' || f.ext_organism AS ext_dam,
       a.db_sex,
       h.ext_code AS ext_sex,
       a.birth_dt,
       a.birth_year,
       a.latitude,
       a.longitude,
       a.db_donor_type,
       i.ext_code AS ext_donor_type,
       a.db_species,
       j.ext_code AS ext_species,
       a.db_breed,
       k.ext_code AS ext_breed,
       a.db_photo,
       a.comment,
       a.la_rep,
       a.la_rep_dt,
       a.last_change_dt,
       a.last_change_user,
       a.dirty,
       a.chk_lvl,
       a.guid,
       a.owner,
       a.version,
       a.synch,
       a.db_hybrid,
       l.ext_code AS ext_hybrid
FROM organisms a LEFT OUTER JOIN transfer b ON a.db_organism = b.db_organism
                 LEFT OUTER JOIN unit c ON b.db_unit = c.db_unit
                 LEFT OUTER JOIN transfer d ON a.db_sire = d.db_organism
                 LEFT OUTER JOIN unit e ON d.db_unit = e.db_unit
                 LEFT OUTER JOIN transfer f ON a.db_dam = f.db_organism
                 LEFT OUTER JOIN unit g ON f.db_unit = g.db_unit
                 LEFT OUTER JOIN codes h ON a.db_sex = h.db_code
                 LEFT OUTER JOIN codes i ON a.db_donor_type = i.db_code
                 LEFT OUTER JOIN codes j ON a.db_species = j.db_code
                 LEFT OUTER JOIN codes k ON a.db_breed = k.db_code
                 LEFT OUTER JOIN codes l ON a.db_hybrid = l.db_code;

-- DROP VIEW v_storage;
CREATE VIEW v_storage AS
SELECT a.guid AS v_guid,
       a.storage_id,
       a.storage_name,
       a.parent_id,
       a.comment,
       a.chk_lvl,
       a.dirty,
       a.guid,
       a.last_change_dt,
       a.last_change_user,
       a.owner,
       a.synch,
       a.version
FROM storage a;

-- DROP VIEW v_protocols;
CREATE VIEW v_protocols AS
SELECT a.guid AS v_guid,
       a.protocol_id,
       a.protocol_name,
       a.comment,
       a.chk_lvl,
       a.dirty,
       a.guid,
       a.last_change_dt,
       a.last_change_user,
       a.owner,
       a.synch,
       a.version,
       a.db_protocol_type,
       b.ext_code AS ext_protocol_type
FROM protocols a LEFT OUTER JOIN codes b ON a.db_protocol_type = b.db_code;

-- DROP VIEW v_projects;
CREATE VIEW v_projects AS
SELECT a.guid AS v_guid,
       a.chk_lvl,
       a.dirty,
       a.guid,
       a.last_change_dt,
       a.last_change_user,
       a.owner,
       a.synch,
       a.version,
       a.project_id,
       a.project_name,
       a.keyword,
       a.begin_dt,
       a.finish_dt,
       a.description,
       a.comment
FROM projects a;

-- DROP VIEW v_samples;
CREATE VIEW v_samples AS
SELECT a.guid AS v_guid,
       a.chk_lvl,
       a.dirty,
       a.guid,
       a.last_change_dt,
       a.last_change_user,
       a.owner,
       a.synch,
       a.version,
       a.sample_id,
       a.ext_sample,
       a.db_organism,
       c.ext_unit || ':::' || c.ext_id || ':::' || b.ext_organism AS ext_organism,
       a.db_vessel_type,
       d.ext_code AS ext_vessel_type,
       a.db_material_type,
       e.ext_code AS ext_material_type,
       a.db_user,
       a.protocol_id,
       f.protocol_id AS ext_protocol_id,
       a.production_dt,
       a.storage_dt,
       a.comment
FROM samples a LEFT OUTER JOIN transfer b ON a.db_organism = b.db_organism
               LEFT OUTER JOIN unit c ON b.db_unit = c.db_unit
               LEFT OUTER JOIN codes d ON a.db_vessel_type = d.db_code
               LEFT OUTER JOIN codes e ON a.db_material_type = e.db_code
               LEFT OUTER JOIN protocols f ON a.protocol_id = f.protocol_id;

-- DROP VIEW v_storage_samples;
CREATE VIEW v_storage_samples AS
SELECT a.guid AS v_guid,
       a.chk_lvl,
       a.dirty,
       a.guid,
       a.last_change_dt,
       a.last_change_user,
       a.owner,
       a.synch,
       a.version,
       a.storage_id,
       a.sample_id,
       a.no_units
FROM storage_samples a;

-- DROP VIEW v_sample_collection;
CREATE VIEW v_sample_collection AS
SELECT a.guid AS v_guid,
       a.chk_lvl,
       a.dirty,
       a.guid,
       a.last_change_dt,
       a.last_change_user,
       a.owner,
       a.synch,
       a.version,
       a.sample_id,
       a.project_id,
       a.reused
FROM sample_collection a;

-- DROP VIEW v_dna_extraction;
CREATE VIEW v_dna_extraction AS
SELECT a.guid AS v_guid,
       a.chk_lvl,
       a.dirty,
       a.guid,
       a.last_change_dt,
       a.last_change_user,
       a.owner,
       a.synch,
       a.version,
       a.dna_id,
       a.sample_id,
       a.project_id,
       a.validation_id,
       a.dna_concentration,
       a.dna_purity,
       a.reused
FROM dna_extraction a;

-- DROP VIEW v_validation;
CREATE VIEW v_validation AS
SELECT a.guid AS v_guid,
       a.validation_id,
       a.gel_id,
       a.protocol_id,
       b.protocol_id AS ext_protocol_id,
       a.comment,
       a.chk_lvl,
       a.dirty,
       a.guid,
       a.last_change_dt,
       a.last_change_user,
       a.owner,
       a.synch,
       a.version
FROM validation a LEFT OUTER JOIN protocols b ON a.protocol_id = b.protocol_id;

-- DROP VIEW v_gel_images;
CREATE VIEW v_gel_images AS
SELECT a.guid AS v_guid,
       a.gel_id,
       a.sample_id,
       a.lane,
       a.valid,
       a.chk_lvl,
       a.dirty,
       a.guid,
       a.last_change_dt,
       a.last_change_user,
       a.owner,
       a.synch,
       a.version
FROM gel_images a;

-- DROP VIEW v_markers;
CREATE VIEW v_markers AS
SELECT a.guid AS v_guid,
       a.marker_id,
       a.marker_name,
       a.db_marker_type,
       b.ext_code AS ext_marker_type,
       a.db_primer1,
       c.ext_code AS ext_primer1,
       a.db_primer2,
       d.ext_code AS ext_primer2,
       a.comment,
       a.chk_lvl,
       a.dirty,
       a.guid,
       a.last_change_dt,
       a.last_change_user,
       a.owner,
       a.synch,
       a.version
FROM markers a LEFT OUTER JOIN codes b ON a.db_marker_type = b.db_code
               LEFT OUTER JOIN codes c ON a.db_primer1 = c.db_code
               LEFT OUTER JOIN codes d ON a.db_primer2 = d.db_code;

-- DROP VIEW v_pcr_amplification;
CREATE VIEW v_pcr_amplification AS
SELECT a.guid AS v_guid,
       a.amplification_id,
       a.ext_amplification,
       a.project_id,
       a.protocol_id,
       b.protocol_id AS ext_protocol_id,
       a.comment,
       a.chk_lvl,
       a.dirty,
       a.guid,
       a.last_change_dt,
       a.last_change_user,
       a.owner,
       a.synch,
       a.version
FROM pcr_amplification a LEFT OUTER JOIN protocols b ON a.protocol_id = b.protocol_id;

-- DROP VIEW v_amplified_samples;
CREATE VIEW v_amplified_samples AS
SELECT a.guid AS v_guid,
       a.amplification_id,
       a.dna_id,
       a.validation_id,
       a.comment,
       a.chk_lvl,
       a.dirty,
       a.guid,
       a.last_change_dt,
       a.last_change_user,
       a.owner,
       a.synch,
       a.version
FROM amplified_samples a;

-- DROP VIEW v_pcr_markers;
CREATE VIEW v_pcr_markers AS
SELECT a.guid AS v_guid,
       a.amplification_id,
       a.marker_id,
       a.chk_lvl,
       a.dirty,
       a.guid,
       a.last_change_dt,
       a.last_change_user,
       a.owner,
       a.synch,
       a.version
FROM pcr_markers a;

-- DROP VIEW v_electrophoresises;
CREATE VIEW v_electrophoresises AS
SELECT a.guid AS v_guid,
       a.electrophoresis_id,
       a.ext_electrophoresis,
       a.project_id,
       a.db_user,
       a.db_software,
       b.ext_code AS ext_software,
       a.product_id,
       a.protocol_id,
       c.protocol_id AS ext_protocol_id,
       a.electrophoresis_dt,
       a.db_method,
       d.ext_code AS ext_method,
       a.purpose,
       e.ext_code AS ext_purpose,
       a.comment,
       a.chk_lvl,
       a.dirty,
       a.guid,
       a.last_change_dt,
       a.last_change_user,
       a.owner,
       a.synch,
       a.version
FROM electrophoresises a LEFT OUTER JOIN codes b ON a.db_software = b.db_code
                         LEFT OUTER JOIN protocols c ON a.protocol_id = c.protocol_id
                         LEFT OUTER JOIN codes d ON a.db_method = d.db_code
                         LEFT OUTER JOIN codes e ON a.purpose = e.db_code;

-- DROP VIEW v_analyzed_samples;
CREATE VIEW v_analyzed_samples AS
SELECT a.guid AS v_guid,
       a.electrophoresis_id,
       a.dna_id,
       a.marker_id,
       a.chk_lvl,
       a.dirty,
       a.guid,
       a.last_change_dt,
       a.last_change_user,
       a.owner,
       a.synch,
       a.version
FROM analyzed_samples a;

-- DROP VIEW v_microsatellites;
CREATE VIEW v_microsatellites AS
SELECT a.guid AS v_guid,
       a.microsatellite_id,
       a.dna_id,
       a.project_id,
       a.marker_id,
       a.allele1,
       a.allele2,
       a.comment,
       a.chk_lvl,
       a.dirty,
       a.guid,
       a.last_change_dt,
       a.last_change_user,
       a.owner,
       a.synch,
       a.version
FROM microsatellites a;

-- DROP VIEW v_sequences;
CREATE VIEW v_sequences AS
SELECT a.guid AS v_guid,
       a.sequence_id,
       a.dna_id,
       a.project_id,
       a.marker_id,
       a.sequence,
       a.comment,
       a.chk_lvl,
       a.dirty,
       a.guid,
       a.last_change_dt,
       a.last_change_user,
       a.owner,
       a.synch,
       a.version
FROM sequences a;


-- DROP VIEW entry_unit;
CREATE VIEW entry_unit AS
SELECT      db_unit, ext_unit, ext_id, db_contact, db_member, opening_dt, closing_dt, last_change_dt, last_change_user, dirty, chk_lvl, guid, owner, version, synch
FROM        unit
WHERE       closing_dt is NULL;

-- DROP VIEW entry_codes;
CREATE VIEW entry_codes AS
SELECT      ext_code, class, db_code, short_name, long_name, description, opening_dt, closing_dt, last_change_dt, last_change_user, dirty, chk_lvl, guid, owner, version, synch
FROM        codes
WHERE       closing_dt is NULL;

-- DROP VIEW entry_transfer;
CREATE VIEW entry_transfer AS
SELECT      db_organism, ext_organism, db_unit, id_set, opening_dt, closing_dt, last_change_dt, last_change_user, dirty, chk_lvl, guid, owner, version, synch
FROM        transfer
WHERE       closing_dt is NULL;

