#!/usr/bin/env perl

BEGIN {
    use Env qw( APIIS_HOME);
    die "\n\tAPIIS_HOME is not set!\n\n" unless $APIIS_HOME;
    push @INC, "$APIIS_HOME/lib";
}

=head1 NAME

index.cgi - Provide a login form

=head1 DESCRIPTION

This script shows a form which allows the web user to provide his/her username
and password to access the main interface of the system.

    +----------------------+
    |  username: ......    |
    |  password: ......    |
    |       <login>        |
    +----------------------+

=cut



##############################################################################
use warnings;
use strict;
use open ':utf8';
use CGI qw ( -no_xhtml :standard :html3);
use HTML::Template;
use Apache::Session::File;
use Apiis;
use Apiis::Init;
use Apiis::Errors;
use Apiis::DataBase::Init;
use Apiis::DataBase::User;
use Apiis::DataBase::Record;
use Apiis::Auth::AR_Common;

Apiis->initialize( VERSION => '$Revision: 1.5 $' );
push @INC, $apiis->project("molabis") . "/lib";
require Labels;


##############################################################################
# INITIALIZING GLOBAL VALUES
##############################################################################
my %session;
my $cgi           = new CGI;
my $sid         = undef;                       # session ID
my $tmpl_dir    = "../../templates/";          # path to HTML templates
my $script_name = "main.cgi";                  # script name for menu
my $session_dir = "/tmp/molab_session_data";   # directory for molab sessions
my $debug_path  = "/tmp/molab_debug_file";     # debug file

##############################################################################
# CREATING A SESSION
##############################################################################
# Create a new session folder if not exist
if ( !-e $session_dir ) {
    eval system("mkdir $session_dir");
    if ($@) { print STDERR "Problems with session directory: $@"; }
}
if ( defined $cgi->param('sid') ) {
    $sid = $cgi->param('sid');
}
eval {
    tie %session, 'Apache::Session::File', $sid, { Directory => $session_dir,};
};
if ($@) { print STDERR "Session data are not accessible: $@"; }

# Read session id for current session
$sid = $session{_session_id};

##############################################################################
# DEBUG
##############################################################################
my $debug = 1;                        # 1: debug, 0: none
if ($debug) { open DBG_FILE, ">>", $debug_path; }
use POSIX qw(strftime);
if ($debug) {
    print DBG_FILE "\n---------------- start of request -----------------";
    print DBG_FILE "\n" . strftime "%a %b %e %H:%M:%S %Y", localtime;
}
print DBG_FILE "\n" . "Generated from login.cgi ...." . $sid
if $debug;

##############################################################################
# GENERATE THE LOGIN FORM
##############################################################################

# open the template file 
my $template = HTML::Template->new(filename => $tmpl_dir . "login.html");

# translate for all labels in the form
my %form_labels = %{ Molab::Labels::get_labels("LOGIN") };
foreach ( $template->param ) {
	if (/^[l|L]\_(.*)/) {
    	if ( $form_labels{$_} ) {	
        	$template->param( $_ => $form_labels{$_} );
        	print DBG_FILE "\n" . $_ . "= " . $form_labels{$_}
			if $debug;        	
        }
	}
}
$template->param('action' => "login");
$template->param('sid' => $sid);

print "Content-Type: text/html; charset=utf-8\n\n", $template->output;

# stop debugging
if ($debug) {
	print DBG_FILE "\n" . strftime "%a %b %e %H:%M:%S %Y", localtime;
    print DBG_FILE "\n--------------- end of request -------------------\n";
}

##############################################################################

=head1 AUTHORS

Truong Van Chi Cong <cong.chi@fli.bund.de>

=cut
