#!/usr/bin/env perl

BEGIN {
    use Env qw( APIIS_HOME);
    die "\n\tAPIIS_HOME is not set!\n\n" unless $APIIS_HOME;
    push @INC, "$APIIS_HOME/lib";
}

=head1 NAME

main.cgi - Administer all web operations in MolabIS

=head1 DESCRIPTION


=cut

##############################################################################
use warnings;
use strict;
use CGI qw ( -no_xhtml :standard :html3);
use CGI::Ajax;
use HTML::Template;
use Apiis;
use Apiis::Init;
use Apiis::Errors;
use Apiis::DataBase::Record;
use Apiis::DataBase::Init;
use Apiis::DataBase::User;
use Apache::Session::File;
use open ':utf8';
use Apiis::Auth::AR_Common;	
use Data::Dumper;
 
Apiis->initialize( VERSION => '$Revision: 1.41 $' );
push @INC, $apiis->project("molabis") . "/lib";
require Labels;
require Main;
require Ajax;

##############################################################################
# INITIALIZING GLOBAL VALUES
##############################################################################
my %session;
my $cgi         = new CGI;
my $sid         = undef;                       	# session ID
my $tmpl_dir    = "../../templates/";          	# path to HTML templates
my $script_name = "main.cgi";                  	# script name for menu
my $session_dir = "/tmp/molab_session_data";   	# directory for molab sessions
my $debug_path  = "/tmp/molab_debug_file";     	# debugging file
my @id;									       	# deprecated - do not use
my $action;     		                       	# action to execute
my $model_file  = "molabis";					# data model file
my $ajax        = undef;                        # ajax string
my $from_path   = "../../reports/";             # where to get report templates
my $to_path     = "/tmp/";                      # where to store outputs (pdf)
my $server_name = $cgi->server_name();			# server name

my $pjx = new CGI::Ajax('external'=> 'main.cgi');

# start debugging
my $debug = 1;                        # 1: debug, 0:non debug
if ($debug) { open DBG_FILE, ">>", $debug_path; }
use POSIX qw(strftime);
my ($begin_time, $end_time);
use Time::HiRes::Value;
$begin_time = Time::HiRes::Value->now();
if ($debug) {
	print DBG_FILE "\n"."--------------- START ---------------";
	print DBG_FILE "\n" . strftime "%b-%e-%Y %H:%M:%S", localtime;
}
print DBG_FILE "\n" . "Generated from main.cgi ...." if $debug;
 
if (defined $cgi->param('sid')){
	print DBG_FILE "\n" . "Session: " . $cgi->param('sid') if $debug;
}
else {
	print DBG_FILE "\n" . "No Session!"	if $debug;
}
if (defined $cgi->param('action')){
	print DBG_FILE "\n" . "Action:" . $cgi->param('action')	if $debug;
}
else {
	print DBG_FILE "\n" . "No Action!" 	if $debug;
}


#############################################################################
# UNICODE HANDLING
##############################################################################

=head2 Unicode Handling

    All data fields which are sent from Web browser must be encoded as Utf8.
    Values of data fields are put in a data hash named $form_input

=cut

# read all form inputs
my @par_names  = $cgi->param;
my $form_input = {};
print DBG_FILE "\n" . "--- BEGIN: Data inputs from browser ---"  if $debug;
if (defined $cgi->param('action')){
	foreach my $name (@par_names) {
		my @val = $cgi->param($name);
		# unicode endcoding
 		foreach (@val) {
 	        $_ = Encode::decode_utf8($_);
         }
	  	if ( scalar @val == 1 ) {
	    	$form_input->{$name} = $val[0];
	  	}
	  	else {
	    	$form_input->{$name} = \@val;
	  	}
	  	print DBG_FILE "\n" . $name . ":" . $form_input->{$name}
	    	if $debug;
	}
}

print DBG_FILE "\n" . "--- END: Data inputs from browser ---"  if $debug;

##############################################################################
# CREATING A SESSION
##############################################################################

=head2 Creating a session

   Create a new session to store all session data for each users. The session
   data will be sent to session directory which is declared in variable
   $session_dir

=cut

if ( defined $form_input->{'sid'} and $form_input->{'sid'} ne "" ) {
    @id = split( '\,', $form_input->{'sid'} );
    $action = $id[1] || "";
}

# when action was sent as form parameter then action from $sid will be
# overwritten by action from form

if ( defined $form_input->{'action'} ) {
    $action = $form_input->{'action'};
}

my $anonymous_session = 0;    	# to make possibility create anonymous link, 
                              	# access via google without session
if ( defined $id[0] and $id[0] ne "" ) {
    $sid = $id[0];
}
else {
    $sid = undef;
    $anonymous_session = 1 if ( $id[0] == -1 );
}

# check if session directory is available
if ( !-e $session_dir ) {
    # if not create it -> can not run without it
    eval system("mkdir $session_dir");
    if ($debug) {
        if ($@) {
            print STDERR "\nProblems with session directory: $@";
            $action = "logout";
        }
        else { print STDERR "\nNew session directory created"; }
    }
}

# create our session
if ( $anonymous_session != 1 ) {
    eval {
        tie %session, 'Apache::Session::File', $id[0],
            { Directory => $session_dir, };
    };

    if ($@) {
        print STDERR "\nSession data are not accessible: $@";
        $action = "logout";
    }
    else {
        $sid = $session{_session_id};    # reading session id into variable
    }
}
else {
    eval {
        tie %session, 'Apache::Session::File', undef,
            { Directory => $session_dir, };
    };

    if ($@) {
        print STDERR "\nSession data are not accessible: $@";
        $action = "logout";
    }
    else {
        $sid = $session{_session_id};    # reading session id into variable
    }
}

if ( $anonymous_session != 1 ) {
    #setting default user anonymous
    $session{username} = 'anonymous' if (defined $form_input->{'username'});
    # check if login name and password are defined and not empty
    if ( $form_input->{'username'} ) {
        $session{username} = $form_input->{'username'}
            if ( $form_input->{'username'} ne "" );
    }
    if ( $form_input->{'password'} ) {
        $session{password} = $form_input->{'password'}
            if ( $form_input->{'password'} ne "" );
    }
}
else {
    $session{username} = "anonymous";
    $session{password} = "pass";
}


##############################################################################
# CHECKING USER AUTHORIZATION
##############################################################################

=head2 Checking User Authorization

	Errors will be sent to user if he gives an invalid username of password.
	On the other hand, the status of success is kept in variable $login_to_db
=cut

my $user_obj;
# creating new APIIS user object
$user_obj = Apiis::DataBase::User->new(
	id => $session{'username'}, user_session_id => $sid
);
$user_obj->password( $session{'password'} );
$apiis->join_model( $model_file, userobj => $user_obj );

# checking if the user can login to the system
unless ( $apiis->status ) {
	if ( $apiis->Auth->user_disabled ) {
		$apiis->status(1);
    	$apiis->errors(
    		Apiis::Errors->new(
        		type      => 'AUTH',
            	severity  => 'CRIT',
            	from      => 'Apiis::Auth::AR_Auth::user_disabled',
            	msg_short => __(
            	"This account was disabled. Please contact administrator."),
        	)
    	);
	}
}
# if the action is 'logout', clear session and quit
if ($action eq 'logout'){
	if ( defined $session{_session_id} ) { 
	  eval {
	    tied(%session)->delete; 
	    untie %session;
	  };
	}
    undef $user_obj if ( defined $user_obj );
    print $cgi->redirect("login.cgi");
}

##############################################################################
# MAKING TEMPLATE
##############################################################################

my %form_labels;
my $template;

if ( $apiis->status ) {  # Logging with errors
	# open the login template file 
	$template = HTML::Template->new(filename => $tmpl_dir . "login.html");
	
	# translate for all labels in the login form
	%form_labels = %{ Molab::Labels::get_labels("LOGIN") };
	foreach ( $template->param ) {
		if (/^[l|L]\_(.*)/) {
	    	if ( $form_labels{$_} ) {	
	        	$template->param( $_ => $form_labels{$_} );
	        	print DBG_FILE "\n" . $_ . "= " . $form_labels{$_}
				if $debug;
	        	
	        }
		}
	}
	$template->param('action' => "login");
	$template->param('sid' => $sid);
	if (defined $cgi->param('sid')){
		$template->param('v_status' => "Invalid username or password");
	}
	else {
		$template->param('v_status' => "Invalid access! Please login ...");
	}
}    
else {					 					# Logging successfully
	# open the main template file
	$template = HTML::Template->new( filename => $tmpl_dir . "main.html" );
	
	# translate for all labels in header
	%form_labels = %{ Molab::Labels::get_labels("HEADER") };
	foreach ( $template->param ) {
		if (/^[l|L]\_(.*)/) {
	    	if ( $form_labels{$_} ) {	
	        	$template->param( $_ => $form_labels{$_} );
	        	print DBG_FILE "\n" . $_ . "= " . $form_labels{$_}
					if $debug;	        	
	        }
		}
	}
	my %main_labels = %{ Molab::Labels::get_labels("MAIN") };
	
	# show the current user in the header section
	$template->param('v_header__user' => $session{username});
	$template->param('v_header__sid' => $sid);
	$template->param('v_header__action' => "logout");
	# which language is used to translate labels
	my $cfg = new Config::IniFiles( -file => "../../etc/apiisrc" );
	if ($debug){
		print DBG_FILE "Language: " . $cfg->val( 'I18N', 'language' )
			if $cfg->val( 'I18N', 'language' );
	}
	my $lang;
	if ($cfg->val( 'I18N', 'language' )) {
	    $lang = $cfg->val( 'I18N', 'language' );
	} else {
		$lang = "en";
	}
	$template->param('v_header__language' => $lang);
	# which part of menu will be seen
	my @tasks = $apiis->Auth->check_system_tasks('www');
	my @menu_tabs = ('0','0','0','0','0','0');
	foreach (@tasks) {
    	if ($_ eq 'workflow')         { $menu_tabs[0] = '1' }
		elsif ($_ eq 'manage_data')   { $menu_tabs[1] = '2' }
		elsif ($_ eq 'reports')       { $menu_tabs[2] = '3' }
		elsif ($_ eq 'export_data')   { $menu_tabs[3] = '4' }
		elsif ($_ eq 'administration'){ $menu_tabs[4] = '5' }
		elsif ($_ eq 'help')          { $menu_tabs[5] = '6' }
		else {}
	}
 	my $menu_str = "";
	for (my $i=0; $i<6; $i++){
    	$menu_str .= $menu_tabs[$i];
	}
	$apiis->log('notice', "menu:".$menu_str);
	
    $template->param('v_header__menu' => $menu_str);
	# which type of banner
	my $banner_type;
	if ($cfg->val( 'INTERFACE', 'banner_type' )) {
	    $banner_type = $cfg->val( 'INTERFACE', 'banner_type' );
	} else {
		$banner_type = "image";
	}
	$template->param('v_header__banner_type' => $banner_type."banner");
  	if ($banner_type eq 'text'){
		$template->param('l_lab_name' => $main_labels{'l_lab_name'});
		$template->param('l_organization_name' => $main_labels{'l_organization_name'});
	}
	
	# BEGIN: handling Ajax actions
	if ($action =~ /^ajax(.*)/){
		$apiis->log( 'notice',"Calling an AJAX action ...");
		my $layout = $form_input->{'layout'};
		my $param_ref;
		my $form_ref;
		my $ext_file;
		my $sql = 
		  Molab::Params::get_search_sql($action,$layout,\%$form_input,$sid);
		$param_ref = Molab::Params::get_input_names($layout);
        $form_ref  = $param_ref->{'form_ref'};
		if ($action eq 'ajax_search'){
			$ajax = &Molab::Ajax::search($sql,$form_ref,$layout);
		}
		elsif ($action eq 'ajax_search_sequences'){
			$ajax = &Molab::Ajax::search_sequences($sql,$form_ref,$layout);
		}
		elsif ($action eq 'ajax_search_microsatellites'){
			$ajax = &Molab::Ajax::search_microsatellites($sql,$form_ref,$layout);
		}
		elsif ($action eq 'ajax_fill'){
			$ajax = &Molab::Ajax::fill_form($layout,$sql,$form_ref);
		}
		# for reuse samples
		elsif ($action eq 'ajax_reuse_samples'){
			$apiis->log( 'notice',"Invoke ajax_reuse_samples()");
			my $project_id = $form_input->{'target_project_loop'};
			my $species = $form_input->{'db_species_loop'};
			my $breed = $form_input->{'db_breed_loop'};
			my $reusing_type = $form_input->{'type_of_reusing_loop'};
			$ajax = &Molab::Ajax::get_samples_for_reusing($layout,'target_samples',$project_id,$species,$breed,$reusing_type);
		}
		# for storage management
		elsif ($action eq "ajax_get_stock"){
            my $sql = "SELECT guid,storage_name,comment,storage_id FROM storage WHERE storage_id=";
            my $stock_id;
            if ($form_input->{'level'} eq '0'){
                $sql = "SELECT storage_id,storage_name,storage_id FROM storage WHERE parent_id=";
                $stock_id = 0;
            }
            elsif ($form_input->{'level'} eq '1'){
                $stock_id = $form_input->{'l1_storage_name_loop'};
            }
            elsif ($form_input->{'level'} eq '2'){
                $stock_id = $form_input->{'l2_storage_name_loop'};
            }
            elsif ($form_input->{'level'} eq '3'){
                $stock_id = $form_input->{'l3_storage_name_loop'};
            }
            elsif ($form_input->{'level'} eq '4'){
                $stock_id = $form_input->{'l4_storage_name_loop'};
            }
            $sql .= "'" . $stock_id . "'";
	    	$sql .= " ORDER BY storage_name";
	    	my $empty_list = '<select id="l'.$form_input->{'level'}.'_storage_name_loop" class="form_input_storage" name="l'.$form_input->{'level'}.'_storage_name_loop">';
            if ($stock_id eq "..."){
                $ajax = "__pjx____pjx____pjx____pjx__$empty_list";
            }
            else{
                $ajax = &Molab::Ajax::get_stock($sql,$form_input->{'level'},$form_input->{'select_id'} );
            }
        }
        # for collecting samples
        elsif ($action eq "ajax_next"){
        	$ajax = &Molab::Ajax::make_form(\%$form_input,\%session,$cgi);
        }
        elsif ($action eq "ajax_upload"){
        	$ajax = &Molab::Ajax::upload_file($cgi);   	
        }
        # prepare samples
        elsif ($action eq "ajax_get_samples"){
        	$apiis->log( 'notice', "call get_sample");
        	$ajax = &Molab::Ajax::get_samples($form_input->{'project_name_loop'},$form_input->{'db_material_type_loop'},$layout,
        	$form_input->{'final_concentration'},$form_input->{'volume_type'},$form_input->{'volume'});   	
        }
        # ajax_list_of_pcrs
        elsif ($action eq "ajax_list_of_pcrs"){
			if ($layout eq "check_pcr_products"){
				$ajax = &Molab::Ajax::list_of_pcrs("pcr_amplification_loop",$form_input->{'project_name_loop'},"validate-selection");
			} else {
	        	$apiis->log( 'notice', "Molab::Ajax::list_of_pcrs:".$form_input->{'filter_1_loop'});
    	    	$ajax = &Molab::Ajax::list_of_pcrs("filter_2_loop",$form_input->{'filter_1_loop'},"search_list_filter");
			}
        }
        # ajax_list_of_electrophoresises
        elsif ($action eq "ajax_list_of_electrophoresises"){
        	$apiis->log( 'notice', "Molab::Ajax::list_of_electrophoresises:".$form_input->{'project_name_loop'});
        	$ajax = &Molab::Ajax::list_of_electrophoresises("ext_electrophoresis_loop",$form_input->{'project_name_loop'});
        }
        # ajax_list_of_markers
        elsif ($action eq "ajax_list_of_markers"){
        	$apiis->log( 'notice', "Molab::Ajax::list_of_markers:".$form_input->{'project_name_loop'});
        	$ajax = &Molab::Ajax::list_of_markers($form_input->{'select_id'},$form_input->{'project_name_loop'},$form_input->{'marker_type'},$layout);
        }
        # prepare samples for electrophoresis
        elsif ($action eq "ajax_get_samples_for_electrophoresis"){
        	$apiis->log( 'notice', "Molab::Ajax::get_samples_for_electrophoresis".$form_input->{'project_name_loop'});
        	$ajax = &Molab::Ajax::get_samples_for_electrophoresis(
        	$form_input->{'project_name_loop'},$form_input->{'pcr_name_loop'},$layout);   	
        }
        # prepare samples for genotyping
        elsif ($action eq "ajax_get_samples_for_genotyping"){
        	$apiis->log( 'notice', "call get_sample_for_genotyping");
        	$ajax = &Molab::Ajax::get_samples_for_genotyping($form_input->{'project_name_loop'},$form_input->{'markers_loop'},$layout);   	
        }
        # prepare dna samples
        elsif ($action eq "ajax_get_dna_samples"){
        	$apiis->log( 'notice', "call get_dna_samples");
        	$ajax = &Molab::Ajax::get_dna_samples($form_input->{'project_name_loop'},$form_input->{'marker_loop'},$layout);   	
        }
        # export data to fasta
        elsif ($action eq "ajax_exporting"){
        	$apiis->log( 'notice', "main.cgi Preparing to export data to files ...");
        	if ($layout eq "ms_toolkit"){
        		$ajax = &Molab::Ajax::export_mstoolkit($layout,"ms",$form_input->{'file_format_loop'},\%$form_input,$server_name);
        	}
        	else{
				my $format = $form_input->{'format'};
        		$ajax = &Molab::Ajax::export_data($layout,$format,"",\%$form_input,$server_name);
        	}
        }
        
        # reporting
        elsif ($action eq "ajax_reporting"){
        	$apiis->log( 'notice', "generating reports"); 
# 			$apiis->log('notice', "DUMP:" . Dumper(%$form_input));
            $ajax = &Molab::Ajax::compile_report(
              $layout,'en','pdf',$from_path,$to_path,
              $form_ref, \%$form_input, $server_name, $session{'username'},$session{'password'})
        }
        
        else {
        	$apiis->log( 'err',"NOT found Ajax action");
        }
	}
	# END: handling Ajax actions
	
	# BEGIN: handling non-Ajax actions
	else {
		$apiis->log( 'notice',"Calling a non-AJAX action ....");
  	    # end filling description hash if it is not in the session
  	    my $content_tmpl;
  	    my $data_session_ref;
  	    my $action_status;
  	    my $df_session_ref;
		($content_tmpl,$data_session_ref,$df_session_ref,$action_status) = 
			Molab::Main::execute(\%$form_input,\%session,$cgi);
		$session{'layout'}    = $data_session_ref->{'layout'};

		$apiis->log("notice", "data_sesison:" . Dumper(%$data_session_ref) );
		
		$template->param( 'v_content' => $content_tmpl->output);
		$template->param( 'v_action_status' => $action_status);
		$session{'errors'}    = $data_session_ref->{'errors'};
		# put all variables in $data_session to the current session
		$apiis->log("notice","<Form Session>");
		for my $key ( keys %$df_session_ref ) {
			my $value = $$df_session_ref{$key};
        	$apiis->log("notice", "\t$key=>$value");
        	$session{$key} = $df_session_ref->{$key};
    	}

    	$apiis->log("notice","</Form Session>");   	
	}
	# END: handling non-Ajax actions
}

##############################################################################
        # FINALL SECTION
##############################################################################

=head2 Final Section

    Code in this section is responsible for showing all web pages and also
    pop-up windows with errors. Some default error messages are replaced here
    or modified to display them correctly in Java Script: window.alert().

=cut

# if there are some errors pop-up window will be shown
if($action ne "login"){
	if (defined $session{'errors'}) {
		$apiis->log( 'info', "Preparing JS alert with errors" );
	    # some characters HAVE TO be changed for window.alert
	    $session{'errors'} =~ tr/'/"/;    # single quots to double
	    my $my_quot = "&quot;";
	    $session{'errors'} =~ s/"/$my_quot/g;
	    my $white_char = ' ';
	    $session{'errors'}
	        =~ s/[\x0A]/$white_char/g;    # unix EOL with white character
	        # change postgreSQL message to something more meaningfull
	    if ( $session{'errors'} =~ /duplicate key violates/ ) {
	    	$session{'errors'} = __(
	            "Duplicate key violation. Record already exists in database"
	        );
	    }
        my $tmp_string = "onload=\"hideHelps();
        window.alert(\'" . $session{'errors'} . "\');initForm(this);\" ";
        
        $template->param( 'on_load_script' => $tmp_string )
            if ( defined $template );
        $apiis->log( "debug", "Alert: " . $tmp_string );
        $session{'errors'} = undef;
	}
	else{ # without errors
		my $tmp_string = "onload=\"hideHelps();initForm(this);\"";
        $template->param( 'on_load_script' => $tmp_string );
	}
}
# stop debugging
if ($debug) {
	$end_time = Time::HiRes::Value->now();
	print DBG_FILE "\n"."processing time: ".$end_time->sub($begin_time). " s";
	print DBG_FILE "\n" . strftime "%b-%e-%Y %H:%M:%S", localtime;
	print DBG_FILE "\n"."--------------- END ---------------\n";
}

# print only results in to the web page in case of AJAX
if ( $action =~ /^ajax(.*)/ ) {
# 	$apiis->log("notice","ajax:".$ajax);
    print "Content-Type: text/html; charset=utf-8\n\n",  $ajax;
}
# print the template to the browser as a HTML page
else {
	print $pjx->build_html( 
          $cgi, $template->output,{-charset=>"UTF-8"});
}

##############################################################################

=head1 AUTHORS
                                                                          
Truong Van Chi Cong <cong.chi@fli.bund.de>

=cut
